/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bernhardjung.dsaprogramm.basis;

import java.util.Comparator;

import de.bernhardjung.dsaprogramm.util.Util;

public interface ElementMitName {

    /**
     * Dieses Suffix wird an die ID des Elements angehaengt.
     * Dadurch erhaelt man eien Schluessel mit dem man das Kuerzel des Elements,
     * so es denn eines gibt, aus der Lokalisierungsdatei laden kann.
     */
    public static final String KUERZEL = ".Kuerzel";
    
    /** 
     * Das Suffix das an die ID des Elements angehaengt wird, um den Schluessel
     * zu erzeugen mit dem der Name des Elements aus der Lokalisierungsdatei
     * geladen werden kann. 
     */
    public static final String NAME = ".Name";
    
    /**
     * Sortiert die Elemente alphabetisch nach Namen.
     */
    public static final Comparator< ElementMitName > NAME_COMPARATOR =
        new Comparator< ElementMitName >() {
            
        public int compare( 
                    final ElementMitName e1, 
                    final ElementMitName e2 ) {

                int ergebnis = Util.compare( e1.getName(), e2.getName() );
                
                if ( 0 != ergebnis ) {
                
                    return ergebnis;
                }
                
                return e1.getId().compareTo( e2.getId() );
            }
    };
        
    /**
     * @return Die Id des Elements.
     */
    public String getId();
    
    /**
     * @return Das Kuerzel fuer den Namen des Elements, falls es ein solches
     *          Kuerzel gibt. Sonst wird irgendein String geliefert.
     */
    public String getKuerzel();
    
    /**
     * @return Der Schluessel unter dem das Kuerzel des Elements in der
     *          Lokalisierungsdatei zu finden ist.
     */
    public String getLokalisierungsschluesselKuerzel();
    
    /**
     * @return Der Schluessel ueber den der Name des Elements aus der 
     *          Lokalisierungsdatei geladen wird.
     */
    public String getLokalisierungsschluesselName();
    
    /**
     * @return Den Namen des Elements.
     */
    public String getName();

    /**
     * @param neuerName des Elements. Es kann {@code null} oder ein leerer 
     *          String uebergeben werden, dann wird versucht einen Namen aus
     *          der Lokalisierungsdatei zu laden. Entspricht der uebergebene
     *          Name der Lokalisierung des Namens diese Elements, so wird
     *          immer die Lokalisierung des Namens verwendet.
     */
    public void setName( String neuerName );
}
