/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basis;

import java.util.ArrayList;
import java.util.List;

import de.bernhardjung.dsaprogramm.gui.DialogFehlermeldung;
import de.bernhardjung.dsaprogramm.util.Util;

public class RuestungWert implements Cloneable {

    /**
     * @param originale Die zu kopierenden Werte.
     * @return Erzeugt eine Deep Copy der Werte.
     */
    public static ArrayList< RuestungWert > deepCopy( 
            final List< RuestungWert > originale ) {
        
        final ArrayList< RuestungWert > kopien = new ArrayList< RuestungWert >(
                originale.size() );
        
        for ( final RuestungWert wert : originale ) {
            
            kopien.add( wert.clone() );
        }
        
        return kopien;
    }
    
    /** Wird an die Observer gesendet, wenn die Behinderung geaendert wird. */
    public static final Object BE = new Object();

    /** Die Behinderung. */
    private int be;

    /** Der Held, der die Ruestung traegt. */
    private Held held;

    /** Der Name der Ruestung. */
    private String name;

    /** Der Ruestungsschutz. */
    private int rs;

    /** Der Typ der Ruestung, um den es sich hier handelt */
    private Ruestung ruestung;

    public RuestungWert() {
        this( new Ruestung() );
    }
    
    public RuestungWert( final Ruestung dieRuestung ) {

        held = null;

        setRuestung( dieRuestung );
    }

    /**
     * @return eine Deep Copy des Wertes.
     */
    @Override
    protected RuestungWert clone() {
        
        RuestungWert clone = null;
        
        try {
            
            clone = (RuestungWert) super.clone();
        
        } catch ( CloneNotSupportedException e ) {
            
            DialogFehlermeldung.zeigeDialog( e );
        }
        
        return clone;
    }
    
    @Override
    public boolean equals( Object obj ) {
        
        if ( null == obj 
                || ! obj.getClass().equals( getClass() ) ) {
            
            return false;
        }
        
        final RuestungWert wert = (RuestungWert) obj;
        
        return Util.equals( 
                be, wert.be,
                name, wert.name,
                rs, wert.rs,
                ruestung, wert.ruestung );
    }
    
    /**
     * @return die Behinderung
     */
    public final int getBe() {
        return be;
    }

    /** @return der Name der Ruestung. */
    public final String getName() {
        return name;
    }

    /** @return den Ruestungsschutz. */
    public final int getRs() {
        return rs;
    }

    /**
     * @return der Typ der Ruestung, um den es sich hier handelt.
     */
    public final Ruestung getRuestung() {
        return ruestung;
    }

    /**
     * Aendert die Behinderung der Ruestung.
     *
     * @param neueBe Die neue Behinderung.
     */
    public final void setBe( final int neueBe ) {

        if ( neueBe != be ) {

            be = neueBe;

            if ( null != held ) {

                held.notifyObservers( BE );
            }
        }
    }

    /**
     * @param derHeld
     *            der die Ruestung traegt. Kann {@code null} sein.
     */
    public final void setHeld( final Held derHeld ) {
        held = derHeld;
    }

    /**
     * Aendert den Namen der Ruestung.
     *
     * @param neuerName fuer die Ruestung.
     */
    public final void setName( final String neuerName ) {
        name = neuerName;
    }

    /**
     * Aendert den Ruestungsschutz der Ruestung.
     *
     * @param neuerRs Der neue Ruestungschutz.
     */
    public final void setRs( final int neuerRs ) {
        rs = neuerRs;
    }

    /**
     * Legt den Typ der Ruestung fest, um den es sich hier handelt. Die
     * Eigenschaften der Ruestung (RS, BE, Gewicht) werden uebernommen.
     *
     * @param neuerTyp
     *            Der Typ fuer diese Ruestung.
     */
    public final void setRuestung( final Ruestung neuerTyp ) {

        if ( null == neuerTyp ) {
            throw new IllegalArgumentException(
                    "Es wurde null uebergeben." );
        }

        ruestung = neuerTyp;

        setName( ruestung.getName() );

        setRs( ruestung.getRs() );

        setBe( ruestung.getBe() );
    }

    @Override
    public String toString() {
        return name;
    }
}
