/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basis.varianten;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.ElementMitName;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Ruestung;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;

public class VariantenRuestungen implements Varianten {

    private final List< Variante > eingefuegte;

    VariantenRuestungen() {

         eingefuegte = new ArrayList< Variante >();
    }

    public void einfuegen( final Variante neueVariante ) {

        if ( neueVariante.isNull() ) {
            
            return;
        }
        
        if ( ! VariantenUtil.enthaeltVarianteMitNamen( 
                getVarianten( null ), 
                neueVariante ) ) { 
            
            eingefuegte.add( neueVariante );
        }
    }

    public Variante getVariante( final Object objekt ) {
        
        if ( objekt instanceof ElementMitName ) {
            
            return new Variante( (ElementMitName) objekt );
        }
        
        final Variante variante;
        
        if ( objekt instanceof String ) {
            
            variante = new Variante( (String) objekt );
        
        } else  {
            
            variante = (Variante) objekt;
        }
        
        return VariantenUtil.getVarianteMitNamen( 
                getVarianten( null ), 
                variante );
    }
    
    public List< Variante > getVarianten( final Held held ) {

        final List< Variante > varianten = new ArrayList< Variante >();

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        for ( final Ruestung ruestung : speicher.getRuestungen() ) {

            if ( ! ruestung.istZusatzruestung() ) {

                varianten.add( new Variante( ruestung ) );
            }
        }

        varianten.addAll( eingefuegte );

        Collections.sort( varianten, Variante.NAME_COMPARATOR );

        return varianten;
    }

    public boolean hatVarianten() {
        return true;
    }

    public boolean istFreieVariante() {
        return true;
    }

}
