/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.bausteine.bonus;

import java.util.List;

import net.n3.nanoxml.IXMLElement;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;

public interface VerbilligteSonderfertigkeit {

    /**
     * Liefert alle Gelaendekunden die ein Held noch nicht besitzt und die
     * noch nicht verbilligt sind.
     */
    public static final VerbilligteSonderfertigkeit GELAENDEKUNDE =
        new VerbilligteSonderfertigkeitGelaendekunde();

    /**
     * @return die GP-Kosten, die einem Baustein durch die verbilligte Sf
     *         entstehen.
     */
    public double getGpKosten();

    /**
     * @param held fuer den die verbilligte Sonderfertigkeit gewaehlt werden
     *      soll.
     * @return die Liste der Sonderfertigkeiten von denen es eine zu waehlen
     *      gilt. Die Liste kann auch aus nur einem Element bestehen. In diesem
     *      Fall gibt es keine Auswahl zu treffen.
     * @throws UnsupportedOperationException wenn es nur eine Sonderfertigkeit
     *      gibt, die ueber {@link #getSonderfertigkeit()} abgerufen werden
     *      muss.
     */
    public List< Sonderfertigkeit > getSonderfertigkeiten( final Held held );

    /**
     * @return die Sonderfertigkeit die verbilligt wird.
     * @throws UnsupportedOperationException wenn es mehrere Sonderfertigkeiten
     *      gibt die ueber {@link #getSonderfertigkeiten(Held)} abgerufen
     *      werden muessen.
     */
    public Sonderfertigkeit getSonderfertigkeit();

    /**
     * @return {@code true} wenn es mehrere Sonderfertigkeiten gibt, von denen
     *      eine als verbilligt zu waehlen ist. Dies koennen ueber
     *      {@link #getSonderfertigkeiten(Held)} abgerufen werden.
     *      Liefert die Methode {@code false} so gibt es nur eine
     *      Sonderfertigkeit die verbilligt wird. Diese kann ueber
     *      {@link #getSonderfertigkeit()} abgerufen werden.
     */
    public boolean hatAuswahl();

    /**
     * @return die XML-Repraesentation des Elements.
     */
    public IXMLElement toXml();
}
