/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bernhardjung.dsaprogramm.daten.factorys;

import static de.bernhardjung.dsaprogramm.daten.xml.NamenlisteXmlTags.ATTR_BEZEICHNUNG;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenlisteXmlTags.ATTR_ID;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenlisteXmlTags.ATTR_LOKALISIERUNGSSCHLUESSEL;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenlisteXmlTags.ATTR_NAME;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import de.bernhardjung.dsaprogramm.basis.namen.Name;
import de.bernhardjung.dsaprogramm.basis.namen.Namenliste;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;

public class FactoryNamenlisten extends Factory< Namenliste > {

    @Override
    public Collection< Namenliste > erzeugen( 
            final IXMLElement wurzelElement ) {
        
        final List< Namenliste > namenlisten = new ArrayList< Namenliste >();
        
        Datenspeicher speicher = Datenspeicher.getInstanz();
        
        for (final IXMLElement namenlisteElement : wurzelElement.getChildren()){
            
            final String id = namenlisteElement.getAttribute( ATTR_ID );
            
            final Namenliste namenliste = speicher.getNamenliste( id );
            namenlisten.add( namenliste );
            
            if ( namenlisteElement.hasAttribute( ATTR_BEZEICHNUNG ) ) {
                
                namenliste.setBezeichnung( 
                        namenlisteElement.getAttribute( ATTR_BEZEICHNUNG ) );
            }
            
            namenliste.setNamen( erzeugeListeVonNamen( namenlisteElement ) );
        }
        
        return namenlisten;
    }

    /**
     * Erzeugt eine Liste von Namen aus ihrere XML-Repraesentation.
     * 
     * @param namenlisteElement
     *          Ein XML-Element das eine Namenliste repraesentiert.
     * @return Die erzeugte Liste von Namen.
     */
    private List< Name > erzeugeListeVonNamen( 
            final IXMLElement namenlisteElement ) {
        
        final List< Name > liste = new ArrayList< Name >();
        
        for ( final IXMLElement nameElement : namenlisteElement.getChildren() ){
            
            final Name name = new Name();
            liste.add( name );
            
            if ( nameElement.hasAttribute( ATTR_LOKALISIERUNGSSCHLUESSEL ) ) {
                
                final String lokalisierungsschluessel =
                    nameElement.getAttribute( ATTR_LOKALISIERUNGSSCHLUESSEL );
                
                name.setLokalisierungsschluessel( lokalisierungsschluessel );
            
            } else {
                
                final String nameString = nameElement.getAttribute( ATTR_NAME );
                
                name.setName( nameString );
            }
        }
        
        return liste;
    }
}
