/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.daten.factorys.bausteine;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_MAENNLICH;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_NAME;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_WEIBLICH;
import net.n3.nanoxml.IXMLElement;

public class FactoryNameProfessionIn {

    /**
     * Ermittel an Hand der Id und der XML-Datei die Namen einer Profession. Der
     * Name wird in 3 Stufen bestimmt. Als erstes wird in der XML-Datei nach
     * einem Namen gesucht. Ist in der Datei kein Name angegeben wird anhand der
     * id ueberprueft ob es einen Lokalisierungstext gibt. Wird einer gefunden,
     * wird er als Name verwendet. Wird kein Lokalisierungstext gefunden wird
     * einfach die Id als Name verwendet.
     *
     * @param wurzelElement
     *            Das Wurzelelement der XML-Datei.
     * @param id
     *            der Profession deren Namen gesucht sind.
     * @return die Bezeichnungen der Profession.
     */
    public final Namen erzeugen(
            final IXMLElement wurzelElement,
            final String id ) {

        final Namen namen = new Namen();

        final IXMLElement nameElement = wurzelElement.getFirstChildNamed( TAG_NAME );

        if ( null != nameElement
          && null != nameElement.getFirstChildNamed( TAG_MAENNLICH ) ) {

            final IXMLElement maennlicherName =
                nameElement.getFirstChildNamed( TAG_MAENNLICH );

            namen.maennlich = maennlicherName.getContent();

        } else {

            namen.maennlich = null;

        }

        if ( null != nameElement
          && null != nameElement.getFirstChildNamed( TAG_WEIBLICH ) ) {

            final IXMLElement weiblicherName =
                nameElement.getFirstChildNamed( TAG_WEIBLICH );

            namen.weiblich = weiblicherName.getContent();

        } else {

            namen.weiblich = null;

        }

        return namen;
    }

    /** Die Namen einer Profession. */
    public static class Namen {

        /** Der Name fuer maennliche Charaktere. */
        public String maennlich;

        /** Der Name fuer weibliche Charaktere. */
        public String weiblich;
    }
}
