/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui;

import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import de.bernhardjung.dsaprogramm.Start;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.net.URI;
import java.net.URL;
import java.util.Calendar;

public class DialogInfo extends JDialog implements Uebersetzbar {

    /**
     * Titel fuer das Tab auf dem die beteiligten Personen aufgelistet werden.
     */
    private static final String BETEILIGTE = "DialogInfo.Beteiligte";

    /**
     * Die Resource-Datei in der die Personen aufgelistet werden, die zujm Projekt beigetragen haben.
     */
    private static final String BETEILIGTE_DATEI = "DialogInfo.Beteiligte.Datei";

    /**
     * Die Datei mit dem Ulisses-Disclaimer, die in der Jar-Datei mitgeliefert
     * wird.
     */
    private static final String DISCLAIMER_DATEI = "DialogInfo.Disclaimer.Datei";

    private static final String COPYRIGHT_BERNHARD_JUNG = 
        "\u00A9 2007 - %1$tY, Bernhard Jung";

    /** Der Text "Lizenziert unter GPL" oder so. */
    private static final String UNTER_GPL = "DialogInfo.UnterGpl";

    /**
     * Die Datei mit der JGoodies Lizenz, die in der Jar-Datei mitgeliefert und
     * auf einem Register im Info-Dialog angezeigt wird.
     */
    private static final String JGOODIES_DATEI = "jgoodies.html";

    /** 
     * Die in der Jar-Datei mitgelieferte Datei mit der GPL, die auf einer
     * Registerkarte im Info-Dialog angezeigt wird. 
     */
    private static final String GPL_DATEI = "gpl.html";

    /** 
     * Gibt den Programmnamen und die Version an, z.B. "DSA Charaktergenerator
     * V 21.1.3" 
     */
    private static final String PROGRAMM_NAME = "Programm.Name";

    /**
     * Der Titel fuer die Registerkarte mit der JGoodies Lizenz.
     */
    private static final String JGOODIES = "DialogInfo.jgoodies";

    /** 
     * Das Kuerzel GPL als Titel fuer die Registerkarte auf der die GPL 
     * angezeigt wird. 
     */
    private static final String GPL = "DialogInfo.GPL";

    /**
     * Der Titel fuer die Registerkarte mit dem Ulisses-Disclaimer.
     */
    private static final String DISCLAIMER = "DialogInfo.Disclaimer";

    /**
     * Der Titel fuer die Registerkarte mit den Lizenzen.
     */
    private static final String LIZENZ = "DialogInfo.Lizenz";

    /**
     * Der Titel fuer die Registerkarte mit den allgemeinen Infos, wie (C) by
     * Bernhard Jung und so.
     */
    private static final String UEBER = "DialogInfo.Ueber";

    private static final String SCHLIESSEN_MNEMONIC = "Knopf.Schliessen.Mnemonic";

    private static final String SCHLIESSEN = "Knopf.Schliessen";

    /** Das Icon mit dem Schriftzug, das die Ueberschrift im Dialog bildet. */
    private static final String UEBERSCHRIFT_ICON = 
        "DialogInfo.Ueberschrift";

    /** Der Link zur Hompage des Programms. */
    private static final String LINK = "DialogInfo.Link";

    /** Meine E-Mail Adresse fuer das Programm. */
    private static final String MAIL = "DialogInfo.Mail";

    private static final String TITEL = "DialogInfo.Titel";

    /** Zeigt die Leute an, die zum Projekt beigetragen haben. */
    private final JEditorPane beteiligteEditorPane;

    /** Zeigt den Ulisses-Disclaimer an. */
    private final JEditorPane disclaimerEditorPane;
    
    /**
     * Das Label mit der Grafik, die die Ueberschrift des Dialogs darstellt.
     * Es zeigt einfach den Namen des Programms in großen Lettern.
     */
    private final JLabel ueberschrift;
    
    private final Uebersetzer uebersetzer;
    
    public DialogInfo( final JFrame hauptfenster ) {
        super( hauptfenster );
        
        setLayout( new FormLayout( "7dlu, F:P:G, 7dlu",
                                   "14dlu, P, 1dlu, F:300px:G, 7dlu, P, 7dlu" ) );

        uebersetzer = new Uebersetzer();
        
        disclaimerEditorPane = erzeugeTextPane( I18n.i18n( DISCLAIMER_DATEI ) );

        beteiligteEditorPane = erzeugeTextPane( I18n.i18n( BETEILIGTE_DATEI ) );

        final JButton schliessen = uebersetzer.erzeugeButton(
                SCHLIESSEN,
                SCHLIESSEN_MNEMONIC );
        schliessen.addActionListener( new ActionListener() {
            public final void actionPerformed( final ActionEvent e ) {
                setVisible( false );
            }
        } );

        final JTabbedPane tabbedPane = erzeugeTabbedPane();
        
        ueberschrift = new JLabel( 
                Util.ladeIcon( I18n.i18n( UEBERSCHRIFT_ICON ) ) );

        final JPanel schliessenPanel = ButtonBarFactory.buildCloseBar( schliessen );

        final CellConstraints cc = new CellConstraints();

        add( ueberschrift, cc.xy( 2, 2 ) );

        add( tabbedPane, cc.xy( 2, 4 ) );

        add( schliessenPanel, cc.xy( 2, 6 ) );
        
        pack();

        setLocationRelativeTo( hauptfenster );
    }


    /**
     * Erzeugt einen Label, der aussieht wie ein Hypertext-Link im Browser,
     * also blau und unterstrichen.
     *
     * @param text
     *            fuer das Label
     * @return ein Label.
     */
    private JLabel erzeugeLinkLabel( final String text ) {

        final JLabel link = new JLabel( "<html><u>" + I18n.i18n( text ) + "</u>");

        link.setForeground( Color.BLUE );

        link.setCursor( Cursor.getPredefinedCursor( Cursor.HAND_CURSOR ) );

        return link;
    }

    /**
     * Erzeugt einen JButton, mit dem der Browser gestaret werden kann, oder
     * ein JLabel da einfach nur eine URL darstellt. Je nachdem ob die
     * Operation {@link Desktop#browse(URI)} unterstuetzt wird.
     *
     * @return ein Label oder ein Button.
     */
    private JLabel erzeugeLink() {

        final JLabel link;

        if ( Desktop.isDesktopSupported()
                && Desktop.getDesktop().isSupported( Desktop.Action.BROWSE ) ) {

            link = erzeugeLinkLabel( LINK );

            link.addMouseListener( new MouseAdapter() {
                public final void mouseClicked( final MouseEvent e ) {
                    try {
                        Desktop.getDesktop().browse( new URI( "http://" + I18n.i18n( LINK ) ) );
                    } catch ( final Exception e1 ) {
                        e1.printStackTrace();
                    }
                }
            } );

        } else {

            link = uebersetzer.erzeugeLabel( LINK );

        }

        return link;
    }

    /**
     * Je nachdem ob {@link Desktop#mail(URI)} unterstuetzt wird erzeugt die
     * Methode einen Knopf der den Mail-Client oeffnet oder ein Label, das eine
     * E-Mail Adresse anzeigt.
     *
     * @return ein Button oder ein Label.
     */
    private JLabel erzeugeMail() {

        final JLabel mail;

        if ( Desktop.isDesktopSupported()
                && Desktop.getDesktop().isSupported( Desktop.Action.MAIL ) ) {

            mail = erzeugeLinkLabel( MAIL );

            mail.addMouseListener( new MouseAdapter() {
                public final void mouseClicked( final MouseEvent e ) {
                    try {
                        Desktop.getDesktop().mail(new URI( "mailto:" + I18n.i18n( MAIL ) ) );
                    } catch ( final Exception e1 ) {
                        e1.printStackTrace();
                    }
                }
            } );

        } else {

            mail = uebersetzer.erzeugeLabel( MAIL );
        }

        return mail;
    }
    
    private JEditorPane erzeugeTextPane() {

        final JEditorPane textPane = new JEditorPane();
        
        textPane.setEditable( false );
        
        return textPane;
    }

    private JEditorPane erzeugeTextPane( final String file ) {

        final JEditorPane textPane = erzeugeTextPane();
        
        final URL textUrl = Start.class.getResource( file );
        try {
            textPane.setPage( textUrl );
        } catch (final IOException e ) {
            e.printStackTrace();
        }
        
        return textPane;
    }
    
    /**
     * @return Das Register mit Copyright Vermerk, Lizenztexten, Disclaimer...
     */
    private JTabbedPane erzeugeTabbedPane() {

        final JTabbedPane tabbedPane = new JTabbedPane();

        uebersetzer.erzeugeTab( 
                tabbedPane, 
                UEBER,
                erzeugePanelUeber() );

        uebersetzer.erzeugeTab( 
                tabbedPane, 
                LIZENZ,
                erzeugeTabLizenzen() );

        uebersetzer.erzeugeTab( 
                tabbedPane, 
                DISCLAIMER,
                Util.erzeugeScrollPane( 
                        disclaimerEditorPane,
                        100,
                        100 ) );

        uebersetzer.erzeugeTab(
                tabbedPane,
                BETEILIGTE,
                Util.erzeugeScrollPane(
                        beteiligteEditorPane,
                        100,
                        100 ) );

        return tabbedPane;
    }

    private JTabbedPane erzeugeTabLizenzen() {
        
        final JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.LEFT );
        
        uebersetzer.erzeugeTab( 
                tabbedPane,
                GPL , 
                Util.erzeugeScrollPane( 
                        erzeugeTextPane( GPL_DATEI ), 
                        100,
                        100 ) );
        
        uebersetzer.erzeugeTab(
                tabbedPane,
                JGOODIES, 
                Util.erzeugeScrollPane( 
                        erzeugeTextPane( JGOODIES_DATEI ),
                        100,
                        100 ) );
        
        return tabbedPane;
    }
    
    /**
     * @return ein Copyright-Vermerk fuer das Programm.
     */
    private JPanel erzeugePanelUeber() {
        
        final JLabel charaktergenerator = uebersetzer.erzeugeLabel( 
                PROGRAMM_NAME );

        final JLabel link = erzeugeLink();
        
        final JLabel lizenz = uebersetzer.erzeugeLabel( UNTER_GPL );
        
        final JLabel mail = erzeugeMail();

        String copyrightText = String.format(
                COPYRIGHT_BERNHARD_JUNG,
                Calendar.getInstance() );
        final JLabel copyright = new JLabel( copyrightText );
        
        final JPanel panel = new JPanel( new FormLayout( 
                "C:P:G", "B:P:G, 7dlu, P, 7dlu, P, 7dlu, P, 7dlu, T:P:G" ) );

        final CellConstraints cc = new CellConstraints();

        panel.add( charaktergenerator, cc.xy( 1, 1 ) );

        panel.add( copyright,          cc.xy( 1, 3 ) );

        panel.add( lizenz,             cc.xy( 1, 5 ) );

        panel.add( link,               cc.xy( 1, 7 ) );

        panel.add( mail,               cc.xy( 1, 9 ) );

        return panel;
    }
    
    public void neuUebersetzen() {

        setTitle( I18n.i18n( TITEL ) );
        
        uebersetzer.neuUebersetzen();

        ueberschrift.setIcon( Util.ladeIcon( I18n.i18n( UEBERSCHRIFT_ICON ) ) );

        neuUebersetzen( disclaimerEditorPane, DISCLAIMER_DATEI );

        neuUebersetzen( beteiligteEditorPane, BETEILIGTE_DATEI );

        pack();
    }

    private void neuUebersetzen( final JEditorPane editorPane, final String datei ) {

        final String resource = I18n.i18n( datei );

        final URL textUrl = Start.class.getResource( resource );

        try {

            editorPane.setPage( textUrl );

        } catch (final IOException e ) {
            e.printStackTrace();
        }
    }
}
