/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.*;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.erschaffung.ErschaffungManager;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeitgruppe;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;

public class PanelSonderfertigkeiten
extends ResetbaresPanel 
implements Uebersetzbar {

    private static final String VERBILLIGTE_REITER = "PanelSonderfertigkeiten.Reiter.verbilligte";

    private static final String GEWAEHLTE_REITER = "PanelSonderfertigkeiten.Reiter.Gewaehlte";

    private static final String VERBILLIGTE = "PanelSonderfertigkeiten.verbilligte";

    private static final String GEWAEHLTE = "PanelSonderfertigkeiten.Gewaehlte";

    private static final String EIGENE_SONDERFERTIGKEIT = "PanelSonderfertigkeiten.EigeneSonderfertigkeit";

    /** Der Text neben der Combobox mit der man die Lernmethode waehlt. */
    private static final String LERNMETHODE =
        "PanelSonderfertigkeiten.Lernmethode";

    /**
     * Die Liste der Sonderfertigkeiten die der Held besitzt, oder besitzen
     * sollte.
     */
    private final PanelSonderfertigkeitenListe gewaehlte;

    /** Der Held, dessen Werte im Panel angezeigt werden. */
    private Held held;

    /** Die Lernmethoden mit denen man die Sfs. aktivieren kann. */
    private final Lernmethoden lernmethoden;

    /**
     * Wird der Held gerade gesteigert?
     * Andernfalls sind wir bei der Erschffung.
     */
    private final boolean steigern;

    /** 
     * Die Liste von Paneln mit den Sonderfertigkeiten aus den 
     * unterschiedlichen Gruppen. 
     */
    private final List< PanelSonderfertigkeitenListe > sonderfertigkeitGruppen;
    
    /** Uebersetzt verschiedenen Label und die Tabbedpane. */
    private final Uebersetzer uebersetzer;
    
    /** Die Liste mit den verbilligten Sonderfertigkeiten. */
    private final PanelSonderfertigkeitenListe verbilligte;
    
    /**
     * Erzeugt das Register mit den einzelnen Listen.
     *
     * @param zumSteigern {@code true}, wenn das Panel beim Steigern eines
     *                    Helden verwendet wird, {@code false}, wenn das Panel
     *                    bei der Heldenerschaffung verwendet wird.
     */
    public PanelSonderfertigkeiten( final boolean zumSteigern ) {

        uebersetzer = new Uebersetzer();
    
        
        
        final JLabel lernmethodeLabel = uebersetzer.erzeugeLabel( 
                LERNMETHODE );
        
        /* Die Combobox mit der man die Lernmethode waehlt. */
        JComboBox lernmethode = uebersetzer.erzeugeComboBox(
                Lernmethode.LEHRMEISTER,
                Lernmethode.SPEZIELLE_ERFAHRUNG,
                Lernmethode.KEINE,
                Lernmethode.FREI );
        resetter.einfuegen( lernmethode );
        
        lernmethoden = new Lernmethoden( lernmethode );

        gewaehlte = new PanelSonderfertigkeitenListe(
                true, zumSteigern, lernmethoden );
        resetter.einfuegen( gewaehlte );

        steigern = zumSteigern;

        final JPanel gewaehltePanel = erzeugePanelGewaehlte();

        verbilligte = new PanelSonderfertigkeitenListe(
                false, zumSteigern, lernmethoden );
        resetter.einfuegen( verbilligte );
        
        final SimpleInternalFrame verbilligteFrame = uebersetzer.erzeugeFrame(
                VERBILLIGTE,
                verbilligte );
        
        final JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.LEFT );
        resetter.einfuegen( tabbedPane );
        
        uebersetzer.erzeugeTab( tabbedPane, GEWAEHLTE_REITER, gewaehltePanel );

        uebersetzer.erzeugeTab( tabbedPane,VERBILLIGTE_REITER,verbilligteFrame);

        final List< Sonderfertigkeitgruppe > gruppen =
            Sonderfertigkeitgruppe.getGruppen();

        sonderfertigkeitGruppen = 
            new ArrayList< PanelSonderfertigkeitenListe >( gruppen.size() );
        
        for ( final Sonderfertigkeitgruppe gruppe : gruppen ) {

            final List< SonderfertigkeitWert > werte =
                    SonderfertigkeitWert.erzeugeWerte( gruppe );

            final PanelSonderfertigkeitenListe p =
                new PanelSonderfertigkeitenListe(
                        false, zumSteigern, lernmethoden );
            resetter.einfuegen(  p  );
            
            p.setWerte( werte );

            sonderfertigkeitGruppen.add( p );

            final SimpleInternalFrame frame = uebersetzer.erzeugeFrame( 
                    gruppe.getLokalisierungsschluesselName(),
                    p );
            
            uebersetzer.erzeugeTab( 
                    tabbedPane,
                    gruppe.getLokalisierungsschluesselKuerzel(), 
                    frame );
        }

        if ( steigern  ) {

            setLayout( new FormLayout( "P:G, P, 2dlu, P, P:G", "2dlu, P, 2dlu, F:P:G" ) );

            final CellConstraints cc = new CellConstraints();

            add( lernmethodeLabel, cc.xy ( 2, 2 ) );

            add( lernmethode, cc.xy( 4, 2 ) );

            add( tabbedPane, cc.xyw( 1, 4, 5 ) );

        } else {

            setLayout( new BorderLayout() );

            add( tabbedPane, BorderLayout.CENTER );
        }
    }

    private JPanel erzeugePanelGewaehlte() {

        JButton button = uebersetzer.erzeugeButton( EIGENE_SONDERFERTIGKEIT );
        button.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                hinzufuegenBenutzerdefinierteSonderfertigkeit();
            }
        } );
        JPanel buttonPanel = new JPanel( new FormLayout( "4dlu, P, 4dlu", "4dlu, P, 4dlu" ) );
        buttonPanel.add( button, new CellConstraints( 2, 2 ) );
        final SimpleInternalFrame gewaehlteFrame = uebersetzer.erzeugeFrame(
                GEWAEHLTE,
                gewaehlte );

        final JPanel panel = new JPanel( new BorderLayout() );
        panel.add( buttonPanel, BorderLayout.NORTH );
        panel.add( gewaehlteFrame );
        return panel;
    }

    private void hinzufuegenBenutzerdefinierteSonderfertigkeit() {

        DialogBenutzerdefinierteSonderfertigkeit dialog = new DialogBenutzerdefinierteSonderfertigkeit();
        dialog.setModal( true );
        dialog.setLocationRelativeTo( this );
        dialog.setVisible( true );
        if ( dialog.isAngenommen() ) {

            SonderfertigkeitWert wert = dialog.getWert();

            if ( steigern ) {
                final int lernkosten =
                        wert.getApKosten( held, lernmethoden.getLernmethode() );

                wert.setLernkosten( lernkosten );
            }

            held.einfuegen( wert );
        }

    }

    public void neuUebersetzen() {

        uebersetzer.neuUebersetzen();
    }
    
    /**
     * Muss zum Steigern eines Helden aufgerufen werden. Bestimmt den Helden
     * der ueber das Panel editiert wird.
     *
     * @param derHeld
     *            der im Panel bearbeitet wird.
     */
    public final void setHeld( final Held derHeld ) {
        
        this.held = derHeld;
        
        gewaehlte.setHeld( derHeld );

        // leitsonderfertigkeiten.setWerte( derHeld.getLeitsonderfertigkeiten() );
        // leitsonderfertigkeiten.setHeld( derHeld );

        final List< SonderfertigkeitWert > verbilligteWerte =
                SonderfertigkeitWert.erzeugeWerte( derHeld.getVerbilligteSonderfertigkeiten() );
        verbilligteWerte.addAll( derHeld.getVerbilligteVarianten() );

        verbilligte.setWerte( verbilligteWerte );

        verbilligte.setHeld( derHeld );

        for ( final PanelSonderfertigkeitenListe p : sonderfertigkeitGruppen ) {

            p.setHeld( derHeld );
        }
    }

    /**
     * Muss bei der Heldenerschaffung aufgerufen werden.
     * Bestimmt den Manager fuer die aktuell Erschaffung und damit
     * auch den Helden dessen Werte ueber die Schnittstelle manipuliert
     * werden.
     *
     * @param manager fuer die aktulle Erschaffung.
     */
    public final void setManager( final ErschaffungManager manager ) {

        held = manager.getHeld();

        gewaehlte.setManager( manager );

        // leitsonderfertigkeiten.setWerte( held.getLeitsonderfertigkeiten() );
        // leitsonderfertigkeiten.setManager( manager );

        for ( final PanelSonderfertigkeitenListe p : sonderfertigkeitGruppen ) {

            p.setManager( manager );
        }

        final List< SonderfertigkeitWert > verbilligteWerte =
                SonderfertigkeitWert.erzeugeWerte( held.getVerbilligteSonderfertigkeiten() );

        verbilligteWerte.addAll( held.getVerbilligteVarianten() );

        verbilligte.setWerte( verbilligteWerte );

        verbilligte.setManager( manager );
    }
}
