/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.action;

import java.net.URL;

import javax.help.HelpBroker;
import javax.help.HelpSet;

import de.bernhardjung.dsaprogramm.Debug;
import de.bernhardjung.dsaprogramm.gui.DialogFehlermeldung;
import de.bernhardjung.dsaprogramm.gui.Hauptpanel;
import de.bernhardjung.dsaprogramm.util.Einstellungen;

public class ActionHilfe extends CharGenAction {

    /** Der Pfad zum Standard Helpset. */
    private static final String HELPSET = 
        "de/bernhardjung/dsaprogramm/hilfe/hilfe.hs";

    /** Repraesentiert den Hilfe-Dialog. */
    private HelpBroker helpBroker; 
    
    public ActionHilfe( final Hauptpanel dasHauptpanel ) {
        
        super( dasHauptpanel,
                "ActionHilfe.Name",
                "hilfe",
                "ActionHilfe.Tasten",
                "ActionHilfe.Mnemonic");
    }
    
    /**
     * Zeigt/Verbirgt den Hilfe Dialog.
     */
    public void actionPerformed() {
        
        if ( null != getHelpBroker() ) {
            
            helpBroker.setDisplayed( ! helpBroker.isDisplayed() );
        }
    }

    public HelpBroker getHelpBroker() {

        if ( null == helpBroker ) {
            
            HelpSet hs = null;
            
            try {
                
                final ClassLoader cl = ActionHilfe.class.getClassLoader();
                
                final URL hsURL = HelpSet.findHelpSet( 
                        cl, 
                        HELPSET,
                        Einstellungen.getInstanz().getSprache()  );
                
                hs = new HelpSet( cl, hsURL );
             
                helpBroker = hs.createHelpBroker();
                
            } catch (Exception ex) {
                
                return null;
            }            
        }
        
        return helpBroker;
    }
    
    @Override
    public void neuUebersetzen() {

        super.neuUebersetzen();
        
        if ( null == helpBroker ) {
            
            return;
        }
        
        try {

            HelpSet hs = null;
                
            final ClassLoader cl = ActionHilfe.class.getClassLoader();
            
            final URL hsURL = HelpSet.findHelpSet( 
                    cl, 
                    HELPSET,
                    Einstellungen.getInstanz().getSprache() );
            
            hs = new HelpSet( cl, hsURL );
         
            helpBroker.setHelpSet( hs );
            
        } catch (Exception ex) {

            ex.printStackTrace();
        }
    }
}
