/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.bonus;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Util;
import de.bernhardjung.dsaprogramm.zauber.Zauber;
import de.bernhardjung.dsaprogramm.zauber.ZauberWert;
import de.bernhardjung.dsaprogramm.zauber.Zaubergruppe;

public class PanelLeitzauber
extends PanelBonus 
implements ActionListener, Uebersetzbar {

    /** Die Anzahl an Zaubern die gewaehlt werden soll. */
    private int anzahlZuWaehlen;
    
    private final List< JCheckBox > checkboxen;

    /**
     * Der Text der angibt, wie viel Leitzauber gewaehlt werden sollen.
     */
    private final JLabel hinweis;
    
    /** Die Zauber die zur Auswahl stehen. */
    private final List< Zauber> zauber;

    public PanelLeitzauber() {

        super( new BorderLayout() );

        anzahlZuWaehlen = 0;
        
        checkboxen = new ArrayList< JCheckBox >();

        hinweis = new JLabel();
        
        zauber = new ArrayList< Zauber >();
    }

    /**
     * Deaktivert die JCheckBoxen, wenn alle Leitzauber gewaehlt wurden.
     */
    public void actionPerformed( final ActionEvent e ) {

        deAktiviereCheckboxen();
    }

    /** Baut die Benutzeroberflaeche neu auf. */
    private void aufbauenBenutzeroberflaeche() {

        removeAll();

        initCheckBoxen();

        // Das Panel auf das die ganzen Elemente kommen.
        final JPanel panel = new JPanel();

        panel.setLayout( erzeugeLayout() );

        final CellConstraints cc = new CellConstraints();

        hinweis.setText( String.format( 
                I18n.i18n( "PanelLeitzauber.Hinweis" ), 
                anzahlZuWaehlen ) );

        panel.add( hinweis, cc.xyw( 2, 2, 3 ) );

        int y = 2;

        for ( int i = 0; i < zauber.size() ; ++i ) {

            y += 2;

            panel.add( checkboxen.get( i ), cc.xy( 3, y ) );
        }

        final JScrollPane scrollPane = Util.erzeugeScrollPane( panel );

        add( scrollPane );
    }

    /**
     * Abhaengig davon ob schon das maximum an Zaubern ausgewahlt wurde
     * deaktivert die Methode die nicht angeklickten Checkboxen, damit man 
     * nicht mehr als die geforderte Anzahl an Zaubern waehlen kann.
     */
    private void deAktiviereCheckboxen() {

        int anzahlGewaehlte = 0;
        
        for ( int i = 0, n = zauber.size() ; i < n ; ++i ) {
            
            if ( checkboxen.get( i ).isSelected() ) {
                
                ++anzahlGewaehlte;
            }
        }
        
        if ( anzahlGewaehlte >= anzahlZuWaehlen ) {

            for ( int i = 0, n = zauber.size() ; i < n ; ++i ) {

                final JCheckBox c = checkboxen.get( i );

                c.setEnabled( c.isSelected() );
            }

        } else {

            for ( int i = 0, n = zauber.size() ; i < n ; ++i ) {

                checkboxen.get( i ).setEnabled( true );
            }
        }
    }

    /**
     * Erzeugt ein Layout um fuer jeden Zauber eine Checkbox darauf zu plazieren.
     * @return
     */
    private FormLayout erzeugeLayout() {

        final StringBuilder zeilen = new StringBuilder();

        zeilen.append( "7dlu, P, " );

        for ( int i = zauber.size() ; i > 0 ; --i ) {

            zeilen.append( "4dlu, P, " );
        }

        zeilen.append( "7dlu" );

        final String spalten = "7dlu, P:G, P, P:G, 7dlu";

        return new FormLayout( spalten, zeilen.toString() );
    }

    /**
     * @return die ausgewaehlten Leitzauber.
     */
    public final List< Zauber > getGewaehlteZauber() {

        final List< Zauber > gewaehlte = 
            new ArrayList< Zauber >( anzahlZuWaehlen );

        for ( int i = 0, n = zauber.size() ; i < n ; ++i ) {

            if ( checkboxen.get( i ).isSelected() ) {

                gewaehlte.add( zauber.get( i ) );
            }
        }

        return gewaehlte;
    }
    
    /**
     * Sort dafuer das fuer jeden Zauber eine Checkbox existiert und
     * setzt den Text der Checkboxen auf die Namen der Zauber.
     */
    private void initCheckBoxen() {

        for ( int i = checkboxen.size(), n = zauber.size(); i < n ; ++i ) {

            final JCheckBox checkbox = new JCheckBox();

            checkbox.addActionListener( this );

            checkboxen.add(  checkbox );
        }

        for ( int i = 0, n = zauber.size() ; i < n ; ++i ) {

            final JCheckBox checkbox = checkboxen.get( i );

            checkbox.setText( zauber.get( i ).getName() );

            checkbox.setEnabled( true );
            
            checkbox.setSelected( false );
        }
    }

    public void neuUebersetzen() {

        hinweis.setText( String.format( 
                I18n.i18n( "PanelLeitzauber.Hinweis" ), 
                anzahlZuWaehlen ) );
        
        List< Zauber > gewaehlte = getGewaehlteZauber();
        
        Collections.sort( zauber, Zauber.NAME_COMPARATOR );
        
        // Alle Checkboxen enablen und mit den neuen Texten versehen.
        initCheckBoxen();
        
        for ( int i = 0, n = zauber.size() ; i < n ; ++i ) {
            
            final Zauber z = zauber.get( i );
            
            final JCheckBox checkBox = checkboxen.get( i );
            
            checkBox.setSelected( gewaehlte.contains( z ) );
        }
        
        deAktiviereCheckboxen();
    }
    
    /**
     * @param held fuer den die Leitzauber gewaehlt werden.
     * @param anzahlLeitzauber die gewaehlt werden soll.
     */
    public final void setDaten( final Held held, final int anzahlLeitzauber ) {

        anzahlZuWaehlen = anzahlLeitzauber;

        zauber.clear();

        for ( final Zauber z : Zaubergruppe.ZAUBER ) {

            final ZauberWert wert;
            
            if ( z.hatVarianten() ) {
                
                Variante variante;
                
                List< Variante > varianten = z.getVarianten( held );
                
                if ( 0 == varianten.size() ) {
                    
                    variante = new Variante();
                
                } else {
                    
                    variante = varianten.get( 0 );
                }
                
                wert = new ZauberWert( z, variante, 0 );
            
            } else {
                
                wert = new ZauberWert( z );
            }

            final KonfliktBeschreibung beschreibung = wert.pruefen( held );

            if ( held.hatLeitzauber( z )
              || beschreibung.enthaeltKonflikt() ) {

                continue;
            }

            zauber.add( z );
        }

        aufbauenBenutzeroberflaeche();

        validate();
        repaint();
    }
}
