package de.bernhardjung.dsaprogramm.gui.bonus;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.bausteine.bonus.TalentbonusFest;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.TalentWert;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

public class PanelTalentBonusFestEntdecker2
        extends PanelTalentBonus
        implements ActionListener, Uebersetzbar {

    private static final String PROF = "PanelTalentBonusFestEntdecker2.Profession";

    private static final String TAW = "PanelTalentBonusFestEntdecker2.TaW";

    private TalentbonusEntdeckerWert bonus;

    private List< JLabel > bonusLabel;

    private FormLayout layout;

    private JLabel profession;

    private List< List< JRadioButton > > radioButtons;

    private JLabel taw;

    private List< JLabel > tawProfLabels;

    private Uebersetzer uebersetzer;

    public PanelTalentBonusFestEntdecker2() {

        bonusLabel = new ArrayList< JLabel >();

        uebersetzer = new Uebersetzer();

        profession = uebersetzer.erzeugeUeberschrift( PROF );

        radioButtons = new ArrayList< List< JRadioButton > >();

        taw  = uebersetzer.erzeugeUeberschrift( TAW );

        tawProfLabels = new ArrayList<JLabel>();
    }

    public void actionPerformed(ActionEvent event ) {

        JRadioButton button = (JRadioButton) event.getSource();
        int zeile = 0;
        int spalte = 0;
        boolean exit = false;
        for ( int anzahlSpalten = radioButtons.size() ; spalte < anzahlSpalten ; ++ spalte ) {
            List< JRadioButton > buttonSpalte = radioButtons.get( spalte );
            int anzahlZeilen = buttonSpalte.size();
            for ( zeile = 0 ; zeile < anzahlZeilen ; ++ zeile )
            {
                if ( buttonSpalte.get( zeile ) == button )
                {
                    exit = true;
                    break;
                }
            }
            if ( exit )
                break;
        }

        bonus.setGewaehltesTalent( spalte, zeile );
        update();
    }

    private  void aufbauenBenutzeroberflaeche() {

        layout = new FormLayout(
                getFormLayoutColumnSpecs(),
                getFormLayoutRowSpecs() );
        setLayout(  layout  );
        CellConstraints cc = new CellConstraints();
        add( taw, cc.xy( 4, 2 ) );
        add( profession, cc.xy( 6, 2 ) );
        int anzahlBonus = bonus.getAnzahlBoni();
        for ( int i = 0 ; i < anzahlBonus ; ++i )
        {
            String text = bonus.getBonusText( i );
            JLabel label = Util.erzeugeUeberschrift( text );
            bonusLabel.add(label);
            add( label, cc.xy( 2 * i + 8, 2 ) );
        }

        int talentAnzahl = bonus.getAnzahlTalente();
        for ( int i = 0 ; i < talentAnzahl ; ++i ) {

            int x = 0;
            int y = 4 + 2 * i;
            Talent talent = bonus.getTalent( i );
            String name = talent.getName();
            JLabel nameLabel = new JLabel( name );
            add( nameLabel, cc.xy( x += 2, y ) );

            String tawString = "-";
            String tawProfString = "-";
            if ( getHeld().hat(talent) )
            {
                int taw = getHeld().getStufe(talent);
                int tawProf = bonus.getTaWZumUmverteilen( talent );
                taw -= tawProf;
                 tawString = String.format( "%1$+d", taw );
                 tawProfString = String.format( "%1$+d", tawProf );
            }
            JLabel tawLabel = new JLabel( tawString );
            add( tawLabel, cc.xy( x += 2, y ) );
            JLabel tawProfLabel  = new JLabel( tawProfString );
            tawProfLabels.add( tawProfLabel );
            add( tawProfLabel, cc.xy( x + 2, y ) );
        }

        for ( int spalte = 0 ; spalte < anzahlBonus ; ++spalte ) {

            int x = 8 + 2 * spalte;
            for ( int zeile= 0 ; zeile < talentAnzahl ; ++zeile ) {

                int y = 4 + 2 * zeile;
                add( erzeugeRadioButton(), cc.xy( x, y ) );
            }
        }

        update();
    }

    private JRadioButton erzeugeRadioButton() {

        JRadioButton button = new JRadioButton();
        button.addActionListener( this );

        if ( radioButtons.isEmpty()
            || radioButtons.get( radioButtons.size() - 1 ).size() == bonus.getAnzahlTalente() )
            radioButtons.add( new ArrayList< JRadioButton >( bonus.getAnzahlTalente() ) );

        radioButtons.get( radioButtons.size() - 1 ).add( button );

        return button;
    }

    private String getFormLayoutColumnSpecs() {

        StringBuilder colSpecs = new StringBuilder();
        colSpecs.append( "7dlu, pref, 4dlu, c:pref, 4dlu, c:pref" );
        int n = bonus.getAnzahlBoni() + bonus.getAnzahlTalente();
        for ( int i = 0; i < n; i++ ) {
            colSpecs.append( ", 4dlu, c:pref" );
        }

        return colSpecs.toString();
    }

    private String getFormLayoutRowSpecs() {

        StringBuilder colSpecs = new StringBuilder();
        colSpecs.append( "7dlu, pref" );
        int n = bonus.getAnzahlTalente();
        for ( int i = 0; i < n; i++ ) {
            colSpecs.append( ", 4dlu, pref" );
        }

        return colSpecs.toString();
    }

    private Held getHeld() {

        return bonus.getHeld();
    }

    @Override
    public List<TalentWert> getTalentBoni() {

        return this.bonus.getTalentBoni();
    }

    public void neuUebersetzen() {

        uebersetzer.neuUebersetzen();

        update();
    }

    /**
     * Bestimmt den Held und den Bonus fuer die eine Auswahl getroffen werden
     * soll.
     *
     * @param derHeld fuer den der Bonus sein soll.
     * @param derBonus der die Auswahl anbietet.
     */
    public final void setBonus( final Held derHeld, final TalentbonusFest derBonus ) {

        bonus = new TalentbonusEntdeckerWert( derHeld, derBonus );

        aufbauenBenutzeroberflaeche();
    }

    private void spaltenzahlSicherstellen() {

        int anzahlBoni = bonus.getAnzahlBoni();
        int anzahlButtons = radioButtons.size();
        CellConstraints cc = new CellConstraints();
        for ( ; anzahlButtons <= anzahlBoni ; ++anzahlButtons )
        {
            int spalte = 8 + 2 * anzahlButtons;
            JLabel label = Util.erzeugeUeberschrift( "?" );
            bonusLabel.add(label);
            add( label, cc.xy( spalte, 2 ) );
            for ( int i = 0, n = bonus.getAnzahlTalente() ; i < n ; ++i )
                add( erzeugeRadioButton(), cc.xy( spalte, 4 + i * 2 ) );
        }
    }

    private void update()
    {
        spaltenzahlSicherstellen();

        int anzahlBoni = bonus.getAnzahlBoni();
        int spalte;
        for ( spalte = 0 ; spalte < anzahlBoni ; ++spalte )
        {
            bonusLabel.get(spalte).setText( bonus.getBonusText( spalte ) );
            bonusLabel.get( spalte ).setVisible(true);
            List< JRadioButton > buttonSpalte = radioButtons.get( spalte );
            int anzahlZeilen = radioButtons.get(radioButtons.size() - 1).size();
            for ( int zeile = 0 ; zeile < anzahlZeilen ; ++zeile )
            {
                if ( buttonSpalte.get( zeile ) != null )
                {
                    buttonSpalte.get( zeile ).setVisible(true);
                    buttonSpalte.get( zeile ).setSelected( bonus.isGewaehlt( spalte, zeile ) );
                }
            }
        }

        for ( int i = 0, n = tawProfLabels.size() ; i < n ; ++i )
        {
            Talent talent = bonus.getTalent( i );
            int tp = bonus.getTaWZumUmverteilen( talent );
            String tawText = "-";
            if ( getHeld().hat(talent) )
            {
                tawText = String.format( "%1$+d", tp );
                //if ( isTpRedistributed( i ) )
                if ( bonus.isTaWUmverteilt( i ) )
                    tawText = "(" + tawText + ")";
            }
            tawProfLabels.get( i ).setText( tawText );
        }

        int anzahlButtons = radioButtons.size();
        for ( ; spalte < anzahlButtons ; ++spalte )
        {
            bonusLabel.get( spalte ).setVisible( false );
            List< JRadioButton > buttonSpalte = radioButtons.get( spalte );
            for ( JRadioButton button : buttonSpalte )
                button.setVisible( false );
        }
    }

    private static class TalentbonusEntdeckerWert {

        private List< Talent > bonus2Quelle;

        private List< Talent > gewaehlt;

        private Held held;

        private List< Talent > talente;

        private List< Integer > tawBoni;

        private List< Integer > tawBoniUmverteilt;

        public TalentbonusEntdeckerWert(
                Held derHeld,
                TalentbonusFest derBonus ) {

            bonus2Quelle = new ArrayList< Talent >();

            gewaehlt = new ArrayList<Talent>();

            held = derHeld;

            talente = derBonus.getTalente( held );

            tawBoni = derBonus.getBoni();

            tawBoniUmverteilt = new ArrayList<Integer>();

            init();
        }

        private void entferneBonus( int bonusIndex ) {

            gewaehlt.remove( bonusIndex );
            bonusIndex -= tawBoni.size();
            bonus2Quelle.remove( bonusIndex );
            tawBoniUmverteilt.remove( bonusIndex );
        }

        private void entferneBonusFuerTalent( Talent talent ) {

            int bonusIndex = bonus2Quelle.indexOf( talent );
            bonusIndex += tawBoni.size();
            entferneBonus( bonusIndex );
        }

        public int getAnzahlBoni() {

            return tawBoni.size() + tawBoniUmverteilt.size();
        }

        public int getAnzahlTalente() {

            return talente.size();
        }

        public int getBonus( int index ) {

            if ( index < tawBoni.size() )
                return tawBoni.get( index );

            index -= tawBoni.size();

            return tawBoniUmverteilt.get(index);
        }

        private int getBonusIndexFuerTalent( Talent talent ) {

            return gewaehlt.indexOf(talent);
        }

        public String getBonusText( int index ) {

            int bonus = getBonus( index );

            return String.format("%1$+d", bonus);
        }

        public Talent getGewaehltesTalent( int bonusIndex ) {

            return gewaehlt.get(bonusIndex);
        }

        public Held getHeld() {

            return held;
        }

        private Talent getQuelltalent( int bonusIndex ) {

            int index = bonusIndex - tawBoni.size();

            if ( 0 <= index && index < bonus2Quelle.size() )
                return bonus2Quelle.get( index );
            else
                return null;
        }

        public Talent getTalent( int index ) {

            return talente.get( index );
        }

        public List<TalentWert> getTalentBoni() {

            final List< TalentWert > talentBoni = new ArrayList< TalentWert >();

            for ( int i = 0, n = gewaehlt.size() ; i < n ; ++i ) {

                final Talent talent = gewaehlt.get( i );

                final int bonus;
                if ( i < tawBoni.size() ) {

                    bonus = tawBoni.get( i );

                } else {

                    bonus = tawBoniUmverteilt.get( i - tawBoni.size() );

                    Talent quelle = bonus2Quelle.get( i - tawBoni.size() );
                    talentBoni.add( new TalentWert( quelle, -bonus ) );
                }

                final TalentWert wert = new TalentWert( talent, bonus );

                talentBoni.add( wert );
            }

            return talentBoni;
        }
        public int getTaWZumUmverteilen( Talent talent ) {

            TalentWert wert = held.getVorgegebenProf( talent );

            if ( wert != null )
                return wert.getUnmodifizierteStufe();
            else
                return 0;
        }

        private boolean hatTawZumUmverteilen( int talentIndex ) {

            return hatTawZumUmverteilen( getTalent( talentIndex ) );
        }

        private boolean hatTawZumUmverteilen( Talent talent ) {

            return 0 != getTaWZumUmverteilen( talent );
        }

        private void hinzufuegenBonusFuerTalent( Talent talent ) {

            int taw = getTaWZumUmverteilen( talent );

            int pos = 0;
            for ( ; pos < tawBoniUmverteilt.size() ; ++pos )
                if ( taw > tawBoniUmverteilt.get( pos ) )
                    break;

            tawBoniUmverteilt.add( pos, taw );
            bonus2Quelle.add( pos, talent );

            pos += tawBoni.size();

            gewaehlt.add( pos, null );
        }

        private void init() {

            for ( int i = 0; i < getAnzahlBoni(); i++ ) {
                gewaehlt.add( null );
            }

            int gewIndex = 0;

            for ( int bonusIndex = 0; bonusIndex < getAnzahlBoni(); bonusIndex++ ) {

                while ( gewIndex + 1 < getAnzahlTalente()
                        && hatTawZumUmverteilen( getTalent( gewIndex ) ) )
                    ++gewIndex;
                if ( gewIndex >= getAnzahlTalente() )
                    break;
                gewaehlt.set( bonusIndex, getTalent( gewIndex ) );
                ++gewIndex;
            }
        }
        
        private boolean isGewaehlt( int talentIndex ) {

            return isGewaehlt( getTalent( talentIndex ) );
        }

        private boolean isGewaehlt( Talent talent ) {

            return gewaehlt.contains( talent );
        }

        public boolean isGewaehlt( int bonusIndex, int talentIndex ) {

            return getGewaehltesTalent( bonusIndex ) == getTalent( talentIndex );
        }

        public boolean isTaWUmverteilt( int indexTalent ) {

            Talent talent = getTalent( indexTalent );

            return bonus2Quelle.contains( talent );
        }

        private boolean isWaehlbar( int talentIndex ) {

            return ! hatTawZumUmverteilen( talentIndex )
                    || bonus2Quelle.contains( getTalent( talentIndex ) );
        }

        public void setGewaehltesTalent(
                int bonusIndex,
                int talentIndexNeu ) {

            Talent talentNeu = getTalent( talentIndexNeu );
            Talent talentAlt = getGewaehltesTalent( bonusIndex );
            int talentIndexAlt = talente.indexOf( talentAlt );

            if ( talentAlt == talentNeu )
                return;

            int bonusMitTalentNeu = getBonusIndexFuerTalent( talentNeu );
            if ( bonusMitTalentNeu != -1 )
                gewaehlt.set( bonusMitTalentNeu, talentAlt );

            gewaehlt.set( bonusIndex, talentNeu );

            if ( bonusMitTalentNeu == -1 ) {

                if( getQuelltalent( bonusIndex ) == talentNeu )
                    entferneBonus( bonusIndex );
                else if ( hatTawZumUmverteilen( talentNeu ) )
                    hinzufuegenBonusFuerTalent( talentNeu );
                if ( hatTawZumUmverteilen( talentIndexAlt ) )
                    entferneBonusFuerTalent( talentAlt );

            } else if ( bonusMitTalentNeu > bonusIndex ) {

                if ( getQuelltalent( bonusMitTalentNeu ) == talentAlt )
                    entferneBonus( bonusMitTalentNeu );
                if ( getQuelltalent( bonusIndex ) == talentNeu )
                    entferneBonus( bonusIndex );

            } else {

                if ( getQuelltalent( bonusIndex ) == talentNeu )
                    entferneBonus( bonusIndex );
                if ( getQuelltalent( bonusMitTalentNeu ) == talentAlt )
                    entferneBonus( bonusMitTalentNeu );
            }

            int pos = -1;

            while ( ( pos = gewaehlt.indexOf( null  ) ) != -1 ) {

                for ( int t = 0; t < getAnzahlTalente(); t++ ) {

                    if ( isWaehlbar( t ) && ! isGewaehlt( t ) ) {

                        gewaehlt.set( pos, getTalent( t ) );
                        break;
                    }
                }
            }

            for ( int i = bonus2Quelle.size() - 1; i >= 0 ; --i ) {

                if ( ! gewaehlt.contains( bonus2Quelle.get( i ) ) )
                    entferneBonusFuerTalent( bonus2Quelle.get( i ) );
            }
        }
    }
}
