/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.namen.Name;
import de.bernhardjung.dsaprogramm.gui.CharGenListModel;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;

class PanelMetaNamenliste extends JPanel implements Uebersetzbar {

    private static final String PREFIX_ID = "Name.";

    /** 
     * Das Textfeld in das ein Schluessel aus der Lokalisierungsdatei eingegeben
     * werden kann.
     */
    private final JTextField lokalisierung;
    
    /**
     * Das Textfeld in das ein Name direkt eingegeben werden kann.
     */
    private final JTextField name;
    
    /** Die Liste der Namen die in dem Panel bearbeitet wird. */
    private final CharGenListModel< Name, Name > namenListModel;
    
    /** Uebersetzt die Texte der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;
    
    public PanelMetaNamenliste() {
        
        uebersetzer = new Uebersetzer();
        
        lokalisierung = new JTextField();
        lokalisierung.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                hinzufuegenLokalisierung();
            }
        } );
        
        name = new JTextField();
        name.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                hinzufuegenName();
            }
        } );
        
        namenListModel = new CharGenListModel< Name, Name >( Name.NAME_COMPARATOR );
        
        aufbauenBenutzeroberflaeche();
    }

    /**
     * Baut die Benutzeroberflaeche auf.
     */
    private void aufbauenBenutzeroberflaeche() {
        
        final String zeilen = "F:P:G";
        
        final String spalten = "0dlu, F:100px:G(0.2), 4dlu, F:P:G(0.8)";

        final FormLayout layout = new FormLayout( spalten, zeilen );
        
        setLayout( layout );
        
        final CellConstraints cc = new CellConstraints();
        
        add( erzeugeLinkeSpalte(), cc.xy( 2, 1 ) );
        
        add( erzeugeRechteSpalte(), cc.xy( 4, 1 ) );
    }
    
    /**
     * @return Die rechte spalte des Editors mit den Textfeldern.
     */
    private JComponent erzeugeRechteSpalte() {

        final JButton lokalisierungHinzufuegen = uebersetzer.erzeugeButton( 
                "PanelMetaNamenliste.Einfuegen" );
        lokalisierungHinzufuegen.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                hinzufuegenLokalisierung();
            }
        });

        final JButton nameHinzufuegen = uebersetzer.erzeugeButton( 
                "PanelMetaNamenliste.Einfuegen" );
        nameHinzufuegen.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                hinzufuegenName();
            }
        });
        
        final JLabel nameUeberschrift = uebersetzer.erzeugeLabel( 
                "PanelMetaNamenliste.Name" );
        
        final JLabel lokalisierungUeberschrift = uebersetzer.erzeugeLabel( 
                "PanelMetaNamenliste.Lokalisierung" );
        
        final String zeilen = "P, 4dlu, P, 4dlu, P, 7dlu, " +
        		              "P, 4dlu, P, 4dlu, P, 0px:G";
        
        final String spalten = "F:P:G";
        
        final FormLayout layout = new FormLayout( spalten, zeilen );
        
        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );
        
        builder.append( lokalisierungUeberschrift );
        builder.nextLine( 2 );
        builder.append( lokalisierung );
        builder.nextLine( 2 );
        builder.append( lokalisierungHinzufuegen );
        
        builder.nextLine( 2 );
        builder.append( nameUeberschrift );
        builder.nextLine( 2 );
        builder.append( name );
        builder.nextLine( 2 );
        builder.append( nameHinzufuegen );
        
        return builder.getPanel();
    }

    /**
     * @return Die linke Spalte des Editors mit der Liste der Namen.
     */
    private JComponent erzeugeLinkeSpalte() {
        
        final JPanel panel = new JPanel( new BorderLayout() );
        
        final JList liste = uebersetzer.erzeugeListe( namenListModel );

        final JButton entfernen = uebersetzer.erzeugeButton( 
                "PanelMetaNamenliste.Entfernen" );
        entfernen.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                namenListModel.removeElementAt( liste.getSelectedIndex() );
            }
        } );
        
        final JScrollPane scrollPane = new JScrollPane( liste );
        
        panel.add( scrollPane, BorderLayout.CENTER );
        
        panel.add( entfernen, BorderLayout.PAGE_END );
        
        return panel;
    }
    
    /**
     * @return Die Liste der eingegebenen Namen.
     */
    public List< Name > getNamen() {
        
        return namenListModel.getElements();
    }
    
    /**
     * Fuegt der Liste der Namen einen neuen Namen hinzu. Der Name wird ueber
     * den Schluessel, der in das Textfeld {@link #lokalisierung} eingegeben
     * wurde initialisiert.
     */
    private void hinzufuegenLokalisierung() {
        
        String schluessel = lokalisierung.getText().trim();
        
        if ( schluessel.equals( "" ) ) {
            
            return;
        }
        
        schluessel = PREFIX_ID + schluessel;
        
        final Name name = new Name();
        
        name.setLokalisierungsschluessel( schluessel );
        
        namenListModel.addElement( name );
    }
    
    /**
     * Fuegt der Liste der Namen einen neuen Namen hinzu. Als Name wird der
     * Text der in das Textfeld {@link #name} eingegeben wurde genutzt.
     */
    private void hinzufuegenName() {

        final String nameString = name.getText().trim();
        
        if ( nameString.equals( "" ) ) {
            
            return;
        }
        
        final Name name = new Name();
        
        name.setName( nameString );
        
        namenListModel.addElement( name );
    }
    
    /**
     * @param namenliste Die Namen die im Editor angezeigt und bearbeitet werden
     *          sollen.
     */
    public void setNamen( final List< Name > namenliste ) {
        
        this.namenListModel.setElements( namenliste );
    }
    
    public void neuUebersetzen() {

        uebersetzer.neuUebersetzen();
    }
}
