/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.ElementMitName;
import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.bausteine.kosten.Kosten;
import de.bernhardjung.dsaprogramm.bausteine.kosten.KostenFest;
import de.bernhardjung.dsaprogramm.bausteine.kosten.KostenRasse;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.gui.CharGenListModel;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

class PanelMetaProfessionDiverses
extends ResetbaresPanel
implements Observer, Uebersetzbar {

    /** Das Textfeld fuer die Id. */
    private final JTextField id;
    
    /**
     * Die Checkbox um anzugeben ob es sich um eine Kaempferprofession handelt.
     */
    private final JCheckBox kaempferprofession;

    /** Der Spinner zum einstellen der Kosten. */
    private final JSpinner kosten;
            
    /**
     * Das Label mit dem die vom Programm berechneten Kosten angezeigt werden.
     */
    private final JLabel berechneteKostenLabel;

    /** Das Textfeld, um den Namen fuer Maenner einzugeben. */
    private final JTextField nameMaennlich;
    

    /** Das Textfeld, um den Namen fuer Frauen einzugeben. */
    private final JTextField nameWeiblich;
    
    /**
     * Die Checkbox mit der angegeben werden kann, dass eine bestimmte Rasse
     * andere Kosten fuer die Profession zahlen muss.
     */
    private final JCheckBox rasseCheckBox;

    /**
     * Die Combobox mit der die Rasse gewaehlt werden kann, die andere Kosten zu
     * zahlen hat.
     */
    private final JComboBox rasseComboBox;

    /** Die Kosten, die die spezielle Rasse zahlen muss. */
    private final JSpinner rasseSpinner;

    
    
    /** Ubersetzt die Texte der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;
    
    /** Der Spinner mit dem sich die Punkte fuer Verbindungen angeben lassen. */
    private final JSpinner verbindungen;
    
    /**
     * Die Checkbox mit der sich angeben laesst, dass die Profession
     * zeitaufwaendig ist.
     */
    private final JCheckBox zeitaufwaendig;

    PanelMetaProfessionDiverses( final PanelMetaProfession panelProfession ) {

        uebersetzer = new Uebersetzer();
    
        
        
        id = new JTextField();
        
        final JLabel idLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaProfessionDiverses.Id" );

        kaempferprofession = uebersetzer.erzeugeCheckBox( 
                "PanelMetaProfessionDiverses.Kaempferprofession" );

        kosten = Util.erzeugeSpinner( 0, -99, 99, 1 );
        
        final JLabel kostenLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaProfessionDiverses.Kosten" );

        berechneteKostenLabel = new JLabel();

        nameMaennlich = new JTextField();
        
        final JLabel nameMaennlichLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaProfessionDiverses.Name.Maennlich" );

        nameWeiblich = new JTextField();
        
        final JLabel nameWeiblichLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaProfessionDiverses.Name.Weiblich" );

        rasseCheckBox = uebersetzer.erzeugeCheckBox( 
                "PanelMetaProfessionDiverses.Rasse" );
        
        final Datenspeicher speicher = Datenspeicher.getInstanz();
        speicher.addObserver( this );

        rasseComboBox = uebersetzer.erzeugeComboBox(
                speicher.getRassen(),
                Rasse.NAME_COMPARATOR );

        rasseSpinner = Util.erzeugeSpinner( 0, 0, 99, 1 );

        verbindungen = Util.erzeugeSpinner( 0, 0, 99, 1 );

        final JLabel verbindungenLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaProfessionDiverses.Verbindungen" );
        
        zeitaufwaendig = uebersetzer.erzeugeCheckBox(
                "PanelMetaProfessionDiverses.Zeitaufwaendig" );

        final JButton kostenBerechnen = uebersetzer.erzeugeButton( 
                "PanelMetaProfessionDiverses.Berechnen" );
        kostenBerechnen.addActionListener( new ActionListener(){
            public void actionPerformed( final ActionEvent e ) {

                final Profession profession = new Profession( "Profession" );

                panelProfession.uebertrageDaten( profession );

                final double kosten = profession.berechneGpKosten();

                berechneteKostenLabel.setText( String.format( "%1$.1f", kosten ) );
            }
        });

        final FormLayout layout =
            new FormLayout( "P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P:G" );

        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );

        builder.setDefaultDialogBorder();

        builder.append( nameMaennlichLabel );
        builder.append( nameMaennlich, 7 );

        builder.append( nameWeiblichLabel );
        builder.append( nameWeiblich, 7 );

        builder.append( idLabel );
        builder.append( id, 7 );

        builder.append( kostenLabel, kosten );
        builder.append( kostenBerechnen, berechneteKostenLabel );

        builder.nextLine();

        builder.append( "", rasseCheckBox );
        builder.append( rasseComboBox, rasseSpinner );

        builder.nextLine();

        builder.append( verbindungenLabel, verbindungen );

        builder.nextLine();

        builder.append( kaempferprofession, 9 );

        builder.append( zeitaufwaendig, 9 );
        
        final JScrollPane scrollPane = 
            Util.erzeugeScrollPane( builder.getPanel() );
        resetter.einfuegen( scrollPane );
        
        setLayout( new BorderLayout() );
        
        add( scrollPane, BorderLayout.CENTER );
    }

    public final void anzeigen( final Profession profession ) {

        anzeigenId( profession.getId() );

        kaempferprofession.setSelected( profession.istKaempferprofession() );

        anzeigenKosten( profession.getKostenObjekt() );

        nameMaennlich.setText( profession.getName() );

        nameWeiblich.setText( profession.getNameWeiblich() );

        verbindungen.setValue( profession.getVerbindungen() );

        zeitaufwaendig.setSelected( profession.istZeitaufwaendig() );
    }

    private void anzeigenId( final String idString ) {

        final int index = idString.indexOf( "." );

        final String suffix = idString.substring( 1 + index );

        id.setText( suffix );
    }
    private void anzeigenKosten( final Kosten k ) {

        if ( k instanceof KostenRasse ) {

            final KostenRasse kostenRasse = (KostenRasse) k;

            kosten.setValue( kostenRasse.getGpKostenNormal() );

            rasseCheckBox.setSelected( true );

            rasseComboBox.setSelectedItem( kostenRasse.getRasse() );

            rasseSpinner.setValue( kostenRasse.getGpKostenRasse() );

        } else {

            kosten.setValue( k.getGpKosten( null ) );

            rasseCheckBox.setSelected( false );

        }
    }

    /** @return die fuer die Profession gewaehlte Id. */
    public String getProfessionId() {
        return "Profession." + id.getText().trim();
    }

    /**
     * @return die maennliche Bezeichnung fuer die Profession.
     */
    public String getProfessionName() {
        return nameMaennlich.getText();
    }
    
    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }

    /** Loescht die Eintraege im Panel. */
    public void eingabenLoeschen() {

        id.setText( "" );

        kaempferprofession.setSelected( false );

        berechneteKostenLabel.setText( "" );

        nameMaennlich.setText( "" );

        nameWeiblich.setText( "" );

        verbindungen.setValue( 0 );

        zeitaufwaendig.setSelected( false );
    }
    
    public final void uebertrageDaten( final Profession profession ) {

        final Kosten kostenObjekt;

        if ( rasseCheckBox.isSelected() ) {

            kostenObjekt = new KostenRasse(
                    (Integer) kosten.getValue(),
                    (Rasse) rasseComboBox.getSelectedItem(),
                    (Integer) rasseSpinner.getValue() );

        } else {

            kostenObjekt = new KostenFest( (Integer) kosten.getValue() );
        }

        profession.setKosten( kostenObjekt );

        profession.setKaempferprofession( kaempferprofession.isSelected() );

        profession.setName( nameMaennlich.getText() );

        profession.setNameWeiblich( nameWeiblich.getText() );

        profession.setVerbindungen( (Integer) verbindungen.getValue() );

        profession.setZeitaufwaendig( zeitaufwaendig.isSelected() );
    }

    @SuppressWarnings("unchecked")
    public void update( final Observable o, final Object arg ) {

        if ( Datenspeicher.RASSEN.equals( arg ) ) {

            final Datenspeicher speicher = Datenspeicher.getInstanz();

            final CharGenListModel< ElementMitName, Rasse > model =
                (CharGenListModel< ElementMitName, Rasse >) rasseComboBox.getModel();
            
            model.setElements( speicher.getRassen() );
        }
    }
}
