/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.util.Beobachter;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Util;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.*;
import java.util.List;

abstract class PanelMetaSonderfertigkeitenAbstrakt
extends ResetbaresPanel
implements Beobachter, Uebersetzbar {

    private final Map< Sonderfertigkeit, List< Integer > > abhaengigkeiten;

    /**
     * Wird aufgerufen, wenn auf eine Checkbox geklickt wird.
     */
    private final ActionListener actionAnAbwaehlen;

    /**
     * Wird aufgerufen, wenn eine Variante gewaehlt wird.
     */
    private final ActionListener actionVariante;

    /**
     * Die Liste der Elemente, die der Benutzer ausgewaehlt hat.
     */
    final BeobachtbareListe< SonderfertigkeitWert > gewaehlte;

    /**
     * Der eigentliche Inhalt des Panels.
     */
    private final JPanel inhalt;
    
    
    
    private final Map< Integer, JComboBox > varianten;

    private final List< JCheckBox > vorNachteile;

    /**
     * Die Liste der Werte die vom Panel angezeigt werden.
     */
    final List< SonderfertigkeitWert > werte;

    /**
     * @param liste Die Liste der Werte, die der Benutzer ausgewaehlt hat.
     */
    PanelMetaSonderfertigkeitenAbstrakt(
            final BeobachtbareListe< SonderfertigkeitWert > liste ) {

        
        
        abhaengigkeiten = new HashMap< Sonderfertigkeit, List<Integer> >();

        actionAnAbwaehlen = new ActionListener() {

            public void actionPerformed( final ActionEvent e ) {

                final JCheckBox checkBox = (JCheckBox) e.getSource();

                final int index = Integer.valueOf( checkBox.getName() );

                wertAnAbgewaehlt( index, checkBox.isSelected() );
            }

        };

        actionVariante = new ActionListener() {

            public void actionPerformed( final ActionEvent e ) {

                final JComboBox comboBox = (JComboBox) e.getSource();

                final int index = Integer.valueOf( comboBox.getName() );

                varianteGewaehlt( index, comboBox.getSelectedItem() );
            }

        };

        gewaehlte = liste;
        gewaehlte.anmelden( this );
        
        inhalt = new JPanel();

        varianten = new HashMap< Integer, JComboBox >();

        vorNachteile = new ArrayList< JCheckBox >();

        werte = new ArrayList< SonderfertigkeitWert >();
        
        final JScrollPane scrollPane = Util.erzeugeScrollPane( inhalt );
        resetter.einfuegen( scrollPane );
        
        setLayout( new BorderLayout() );
        
        add( scrollPane, BorderLayout.CENTER );
    }

    void aktualisieren( final int index ) {

        final SonderfertigkeitWert wert = werte.get( index );

        final JCheckBox checkBox = vorNachteile.get( index );

        checkBox.setSelected( gewaehlte.contains( wert ) );

        if ( varianten.containsKey( index ) ) {

            varianten.get( index ).setSelectedItem( wert.getVariante() );
        }
    }

    void aktualisieren( final Sonderfertigkeit sonderfertigkeit ) {

        if ( abhaengigkeiten.containsKey( sonderfertigkeit ) ) {

            for ( final int index : abhaengigkeiten.get( sonderfertigkeit ) ) {

                aktualisieren( index );
            }
        }
    }

    void aufbauenBenutzeroberflaeche() {

        inhalt.removeAll();

        varianten.clear();
        vorNachteile.clear();

        initAbhaengigkeiten();

        // Das neue Layout festlegen.
        final GroupLayout layout = new GroupLayout( inhalt );

        layout.setAutoCreateContainerGaps( true );

        layout.setAutoCreateGaps( true );

        inhalt.setLayout( layout );

        final GroupLayout.ParallelGroup hGroup = layout.createParallelGroup();

        layout.setHorizontalGroup( hGroup );

        final GroupLayout.SequentialGroup vGroup = layout.createSequentialGroup();

        layout.setVerticalGroup( vGroup );

        // Alle 5 Zeilen wird ein Trennstrich eingefuegt, das erleichtert dem
        // Benutzer die Orientierung in den Zeilen der Liste.
        final int extraZeilen = 1 + ( werte.size() / 4 );

        // Jetzt die Liste Zeile fuer Zeile aufbauen.
        for ( int i = 0, n = werte.size() + extraZeilen, trennstriche = 0
                ; i < n
                ; ++i ) {

            final GroupLayout.SequentialGroup zeileH = layout.createSequentialGroup();
            final GroupLayout.ParallelGroup zeileV =
                layout.createParallelGroup( GroupLayout.Alignment.CENTER, false );

            if ( ( i % 5 ) == 0 ) {
                // ein Trennstrich wird eingefuegt.

                final JSeparator separator = new JSeparator();

                zeileH.addComponent( separator );
                zeileV.addComponent( separator );

                hGroup.addGroup( zeileH );
                vGroup.addGroup( zeileV );

                ++trennstriche;

                continue;
            }

            // Das ist der index des Wertes in werte, der in der aktuellen
            // Zeile dargestellt wird.
            final int index = i - trennstriche;

            final String indexString = String.valueOf( index );

            final SonderfertigkeitWert wert = werte.get( index );

            final Sonderfertigkeit sonderfertigkeit = wert.getSonderfertigkeit();

            // Spalte mit der Checkbox und evtl mit der Combobox aufbauen.
            if ( sonderfertigkeit.hatVarianten() ) {

                // Hat der Wert eine Variante wird neben der Checkbox
                // eine Combobox benoetigt um die Variante waehlen zu koennen.

                final JCheckBox checkBox = new JCheckBox();

                checkBox.setText( sonderfertigkeit.getName() );

                checkBox.setSelected( gewaehlte.contains( wert ) );

                checkBox.setName( indexString );

                checkBox.addActionListener( actionAnAbwaehlen );

                vorNachteile.add( checkBox );

                zeileH.addComponent( checkBox );
                zeileV.addComponent( checkBox );

                final Vector< Object > variantenVector =
                    new Vector< Object >( sonderfertigkeit.getVarianten( null ) );
                final JComboBox comboBox =
                    new JComboBox( variantenVector );

                comboBox.setEditable( sonderfertigkeit.hatFreieVarianten() );

                comboBox.setSelectedItem( wert.getVariante() );

                comboBox.addActionListener( actionVariante );

                varianten.put( index, comboBox );
                comboBox.setName( indexString );

                zeileH.addComponent( comboBox, 100, 100, Short.MAX_VALUE );
                zeileV.addComponent( comboBox );

            } else {

                final JCheckBox checkBox = new JCheckBox( wert.getName() );

                checkBox.setSelected( gewaehlte.contains( wert ) );

                checkBox.setName( indexString );

                checkBox.addActionListener( actionAnAbwaehlen );

                vorNachteile.add( checkBox );

                zeileH.addComponent( checkBox,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        Short.MAX_VALUE );
                zeileV.addComponent( checkBox );

            }

            hGroup.addGroup( zeileH );
            vGroup.addGroup( zeileV );
        }

    }

    abstract void wertAnAbgewaehlt( final int index, final boolean gewaehlt );

    private void initAbhaengigkeiten() {

        abhaengigkeiten.clear();

        for ( int i = 0, n = werte.size(); i < n ; ++i ) {

            final Sonderfertigkeit sonderfertigkeit =
                werte.get( i ).getSonderfertigkeit();

            if ( ! abhaengigkeiten.containsKey( sonderfertigkeit ) ) {

                abhaengigkeiten.put( sonderfertigkeit, new ArrayList< Integer >( 2 ) );
            }

            abhaengigkeiten.get( sonderfertigkeit ).add( i );
        }
    }

    /**
     * Sortiert die Liste der Sonderfertigkeiten neu und aktualisiert die 
     * angezeigten Texte. 
     */
    public void neuUebersetzen() {
        
        Collections.sort( werte, SonderfertigkeitWert.NAME_COMPARATOR );
        
        aufbauenBenutzeroberflaeche();
    }
    
    /**
     * Wird aufgerufen, wenn in einer Combobox einen Variante gewaehlt wird.
     *
     * @param index
     *            des Wertes fuer den die Variante gewaehlt wurde.
     * @param variante
     *            die gewaehlt wurde.
     */
    public final void varianteGewaehlt( final int index, final Object variante ) {

        if ( null != variante ) {

            final SonderfertigkeitWert wert = werte.get( index );

            wert.setVariante( variante );
        }

        aktualisieren( index );
    }
}
