/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collections;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.bausteine.Heldenbaustein;
import de.bernhardjung.dsaprogramm.bausteine.bonus.Talentbonus;
import de.bernhardjung.dsaprogramm.gui.CharGenListModel;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;

class PanelMetaTalentboni 
extends ResetbaresPanel 
implements ListSelectionListener, Uebersetzbar {

    enum Modus { KULTUR, PROFESSION, RASSE }

    /**
     * Der Reitertext fuer das Tab, auf dem man den speziellen Talentbonus auf
     * die alten Sprachen einstellen kann.
     */
    private static final String ALTE_SPRACHE =
        "PanelMetaTalentboni.AlteSprache";

    /**
     * Der Reitertext fuer das Tab, auf dem man die festen Talentboni einstellen
     * kann.
     */
    private static final String BONUS =
        "PanelMetaTalentboni.Bonus";

    /**
     * Der Reitertext fuer das Tab, auf dem man die festen Talentboni einstellen
     * kann.
     */
    private static final String BONUS_2 =
        "PanelMetaTalentboni.Bonus2";

    /** Die Aufschrift des Knopfes, zum loeschen eines Bonus. */
    private static final String ENTFERNEN =
        "PanelMetaTalentboni.Entfernen";

    /** Der Reitertext. */
    private static final String FREMDSPRACHEN =
        "PanelMetaTalentboni.Fremdsprachen";

    /**
     * Der Reitertext fuer das Tab, auf dem man einen Talentbonus zum freien
     * Verteilen erzeugen kann.
     */
    private static final String VERTEILEN =
        "PanelMetaTalentboni.Verteilen";

    /** Der besondere Bonus auf Bosparano und Ur-Tulamidya. */
    private PanelMetaTalentbonusFestAlteSprache alteSprache;

    /** Fuer Boni wie: "Singen oder Tanzen +3, das andere +1" */
    private final PanelMetaTalentbonusFest bonus;

    /** Die unterschiedlichen Boni, die es bei einer Profession gibt. */
    private PanelMetaTalentbonusFestProfession bonus2;

    /** Fuer einen Bonus auf Fremdsprachen bei einer Kultur. */
    private PanelMetaTalentbonusFestKultur fremdsprachen;

    
    
    /**
     * Eine Liste aller bisher gewaehlten Talentboni.
     */
    private final JList talentboniListe;

    /** Das ListModel fuer die Liste {@link #talentboniListe}. */
    private final CharGenListModel< Talentbonus, Talentbonus > talentboniModel;

    /**
     * Zeigt den in der Liste ({@link #talentboniModel}) gewaehlten Bonus an.
     */
    private final JTextArea talentbonusAnzeige;

    /**
     * Uebersetzt die Texte der Benutzeroberflaeche neu.
     */
    private final Uebersetzer uebersetzer;
    
    /** Fuer Boni wie: "4 Punkte auf Singe und Tanzen verteilen". */
    private final PanelMetaTalentbonusFrei verteilen;

    PanelMetaTalentboni( final Modus modus ) {

        uebersetzer = new Uebersetzer();
    
        
        
        talentboniModel = new CharGenListModel< Talentbonus, Talentbonus >();
        uebersetzer.einfuegen( talentboniModel );

        talentboniListe = uebersetzer.erzeugeListe( talentboniModel );
        talentboniListe.addListSelectionListener( this );
        talentboniListe.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );

        talentbonusAnzeige = new JTextArea();
        talentbonusAnzeige.setLineWrap( true );
        talentbonusAnzeige.setWrapStyleWord( true );
        talentbonusAnzeige.setEditable( false );

        bonus = new PanelMetaTalentbonusFest( talentboniModel, modus );
        resetter.einfuegen( bonus );
        
        if ( modus == Modus.KULTUR ) {

            fremdsprachen = new PanelMetaTalentbonusFestKultur( talentboniModel );
            resetter.einfuegen( fremdsprachen );

        } else if ( modus == Modus.PROFESSION ) {

            bonus2 = new PanelMetaTalentbonusFestProfession( talentboniModel );
            resetter.einfuegen( bonus2 );

            alteSprache = new PanelMetaTalentbonusFestAlteSprache( talentboniModel );
            resetter.einfuegen( alteSprache );
        }

        verteilen = new PanelMetaTalentbonusFrei( talentboniModel );
        resetter.einfuegen( verteilen );

        aufbauenBenutzeroberflaeche( modus );
    }

    public final void anzeigen( final Heldenbaustein baustein ) {

        talentboniListe.clearSelection();

        final List< Talentbonus > talentboni = baustein.getTalentboni();

        Collections.sort( talentboni );

        talentboniModel.setElements( talentboni );
    }

    private void aufbauenBenutzeroberflaeche( final Modus modus ) {
        
        final JScrollPane talentbonusAnzeigeScrollPane = 
            new JScrollPane( talentbonusAnzeige );
        resetter.einfuegen( talentbonusAnzeigeScrollPane );
        
        final JScrollPane talentboniListeScrollPane = 
            new JScrollPane( talentboniListe );
        resetter.einfuegen( talentboniListeScrollPane );
        
        final JButton entfernen = uebersetzer.erzeugeButton( ENTFERNEN );
        entfernen.addActionListener( new ActionListener(){
            public void actionPerformed( final ActionEvent e ) {
                entfernen( e );
            }
        });

        final JTabbedPane tabbedPane = new JTabbedPane();
        resetter.einfuegen( tabbedPane );

        uebersetzer.erzeugeTab( tabbedPane, BONUS, bonus );

        if ( modus == Modus.KULTUR ) {

            uebersetzer.erzeugeTab( tabbedPane, FREMDSPRACHEN, fremdsprachen );

        } else if ( modus == Modus.PROFESSION ) {

            uebersetzer.erzeugeTab( tabbedPane, BONUS_2, bonus2 );

            uebersetzer.erzeugeTab( tabbedPane, ALTE_SPRACHE, alteSprache );
        }

        uebersetzer.erzeugeTab( tabbedPane, VERTEILEN, verteilen );

        setLayout( new FormLayout( "7dlu, F:75px:G, 4dlu, F:P:G, 7dlu",
            "7dlu, F:50px:G(0.2), 4dlu, F:100px:G(0.8), 4dlu, P, 7dlu"));

        final CellConstraints cc = new CellConstraints();

        add( talentbonusAnzeigeScrollPane, cc.xy( 2, 2 ) );

        add( talentboniListeScrollPane, cc.xy( 2, 4 ) );

        add( entfernen, cc.xy( 2, 6 ) );

        add( tabbedPane, cc.xywh( 4, 2, 1, 5 ) );
    }

    /**
     * Wird aufgerufen, wenn der gewaehlte Bonus aus der Liste
     * {@link #talentboniModel} entfernt werden soll.
     */
    private void entfernen( final ActionEvent e ) {

        final int index = talentboniListe.getSelectedIndex();

        if ( index >= 0 ) {

            talentboniModel.removeElementAt( index );
        }
    }

    /**
     * @return eine Liste der Talentboni.
     */
    public List< Talentbonus > getBoni() {
        return talentboniModel.getElements();
    }

    /** Uebersetzt die Texte auf dem Panel neu. */
    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }
    
    /** Loescht alle gewaehlten Talentboni. */
    public void eingabenLoeschen() {
        talentboniModel.clear();

        bonus.eingabenLoeschen();

        verteilen.eingabenLoeschen();

        if ( null != fremdsprachen ) {

            fremdsprachen.eingabenLoeschen();

        } else if ( null != bonus2 ) {

            bonus2.eingabenLoeschen();
        }
    }
    
    /**
     * Wird aufgerufen wenn in der Liste {@link #talentboniModel} ein Talentbonus
     * gewaehlt wird.
     *
     * @param e
     */
    public void valueChanged( final ListSelectionEvent e ) {

        if ( e.getValueIsAdjusting() ) {

            return;
        }

        final Object gewaehlt = talentboniListe.getSelectedValue();

        if ( null != gewaehlt ) {

            talentbonusAnzeige.setText( gewaehlt.toString() );

        } else {

            talentbonusAnzeige.setText( "" );
        }
    }

}
