/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.ElementMitName;
import de.bernhardjung.dsaprogramm.bausteine.bonus.Talentbonus;
import de.bernhardjung.dsaprogramm.bausteine.bonus.TalentbonusFest;
import de.bernhardjung.dsaprogramm.bausteine.bonus.TalentbonusTyp;
import de.bernhardjung.dsaprogramm.bausteine.bonus.talentlisten.TalentListeFest;
import de.bernhardjung.dsaprogramm.gui.CharGenListModel;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.gui.meta.PanelMetaTalentboni.Modus;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.util.ComparatorInteger;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

class PanelMetaTalentbonusFest 
extends ResetbaresPanel 
implements Uebersetzbar {

    /**
     * Die Ueberschrift fuer den Abschnitt in dem die Stufenboni gewaehlt
     * werden.
     */
    private static final String BONI =
        "PanelMetaTalentbonusFest.Boni";

    /** Der Text auf dem Knopf mit dem ein Stufenbonus hinzugefuegt wird. */
    private static final String EINFUEGEN =
        "PanelMetaTalentbonusFest.Einfuegen";

    /**
     * Der Text der Checkbox, mit der man angibt, dass es sich um einen
     * Entdecker-Bonus handelt.
     */
    private static final String ENTDECKER =
        "PanelMetaTalentbonusFest.Entdecker";

    /**
     * Der Text der Checkbox mit der angegeben werden kann, dass der Held das
     * Talent als Leittalent erhalten soll.
     */
    private static final String LEITTALENT =
        "PanelMetaTalentbonusFest.Leittalent";

    /**
     * Die Ueberschrift fuer den Abschnitt in dem die Talente gewaehlt
     * werden.
     */
    private static final String TALENTE =
        "PanelMetaTalentbonusFest.Talente";

    /**
     * Der Text auf dem Knopf, mit dem man den im Panel gewaehlten Bonus, in die
     * Liste der Talentboni uebernimmt.
     */
    private static final String UEBERNEHMEN =
        "PanelMetaTalentbonusFest.Uebernehmen";

    /**
     * Der Tooltip, der angezeigt wird, wenn mehr Boni als Talente gewaehlt
     * sind.
     */
    private static final String ZU_VIELE_BONI =
        "PanelMetaTalentbonusFest.ZuVieleBoni";

    /** Ein Spinner, in dem ein Stufenbonus eingestellt werden kann., */
    private final JSpinner bonus;

    /**
     * Die Checkbox mit der man angibt, dass die Sonderregel fuer den Entdecker
     * angewendet werden soll.
     */
    private final JCheckBox entdecker;

    /** Die vom Benutzer, fuer den Bonus, ausgewaehlten Stufenboni. */
    private final JList gewaehlteBoniListe;

    /** Das ListModel fuer die Liste der Stufenboni. */
    private final CharGenListModel< Integer, Integer > gewaehlteBoniModel;

    /** Die Liste der fuer den Bonus gewaehlten Talente. */
    private final JList gewaehlteTalenteListe;
    
    /** Die vom Benutzer, fuer den Bonus, ausgewaehlten Talente. */
    private final 
    CharGenListModel< ElementMitName, Talent > gewaehlteTalenteModel;

    /**
     * Mit dieser Checkbox kann angegeben werden, ob es sich bei dem erhaltenen
     * Talent um ein Leittalent handelt.
     */
    private final JCheckBox leittalent;

    
    
    /** Eine Liste aller Talente. */
    private final JList talente;

    /** Die Liste von Talentboni, in die man, ueber dieses Panel, neue Talentboni einfuegt. */
    private final CharGenListModel< Talentbonus, Talentbonus > talentboni;

    /**
     * Der Knopf mit dem man einen im Panel eingestellten Talentbonus in die
     * Liste der Talentboni uebernimmt.
     */
    private final JButton uebernehmen;

    /** Uebersetzt die Texte der Benutzeroberflaeche neu. */
    private final Uebersetzer uebersetzer;
    
    PanelMetaTalentbonusFest(
            final CharGenListModel< Talentbonus, Talentbonus > dieTalentboni,
            final Modus modus ) {

        uebersetzer = new Uebersetzer();

        
        
        bonus = Util.erzeugeSpinner( 0, -9, 99, 1 );

        entdecker = uebersetzer.erzeugeCheckBox( ENTDECKER );

        gewaehlteBoniModel = new CharGenListModel< Integer, Integer >(
                new ComparatorInteger() );
        
        gewaehlteBoniListe = new JList( gewaehlteBoniModel );
        gewaehlteBoniListe.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        gewaehlteBoniListe.addListSelectionListener(
                new ListSelectionListener() {
                    public void valueChanged( final ListSelectionEvent e ) {
                        if ( e.getValueIsAdjusting() ) {
                            return;
                        }
                        final int index = gewaehlteBoniListe.getSelectedIndex();
                        if ( index >= 0 ) {
                            gewaehlteBoniModel.removeElementAt( index );
                            pruefen();
                        }
                    }
                });

        gewaehlteTalenteModel = new CharGenListModel< ElementMitName, Talent >(
                Talent.NAME_COMPARATOR );

        gewaehlteTalenteListe = 
            uebersetzer.erzeugeListe( gewaehlteTalenteModel );
        gewaehlteTalenteListe.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        gewaehlteTalenteListe.addListSelectionListener(
                new ListSelectionListener(){
                    public void valueChanged( final ListSelectionEvent e ) {
                        if ( e.getValueIsAdjusting() ) {
                            return;
                        }
                        final int index = gewaehlteTalenteListe.getSelectedIndex();
                        if ( index >= 0 ) {
                            gewaehlteTalenteModel.removeElementAt( index );
                            pruefen();
                        }
                    }
                });

        leittalent = uebersetzer.erzeugeCheckBox( LEITTALENT );

        final List< Talent > talentListe = Talent.getTalente();
        talentListe.addAll( Talentgruppe.GABEN );
        talente = uebersetzer.erzeugeListe(
                talentListe,
                Talent.TALENTGRUPPEN_COMPARATOR );
        talente.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        talente.addMouseListener( new MouseAdapter() {
            @Override
            public void mouseClicked( final MouseEvent e ) {
                final Object talent = talente.getSelectedValue();
                if ( null != talent ) {
                    if ( ! gewaehlteTalenteModel.contains( (Talent) talent ) ) {
                        gewaehlteTalenteModel.addElement( talent );
                        pruefen();
                    }
                }
            }
        } );

        talentboni = dieTalentboni;

        uebernehmen = uebersetzer.erzeugeButton( UEBERNEHMEN );
        uebernehmen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {

                final TalentbonusFest bonus = new TalentbonusFest(
                        new TalentListeFest(gewaehlteTalenteModel.getElements()),
                        gewaehlteBoniModel.getElements() );

                bonus.setLeittalent( leittalent.isSelected() );

                if ( entdecker.isSelected() ) {

                    bonus.setTyp( TalentbonusTyp.ENTDECKER );
                }

                talentboni.addElement( bonus );

                gewaehlteBoniModel.clear();
                gewaehlteTalenteModel.clear();

                pruefen();
            }
        });
        uebernehmen.setEnabled( false );

        aufbauenBenutzeroberflaeche( modus );
    }

    private void aufbauenBenutzeroberflaeche( final Modus modus ) {

        final JButton einfuegen = uebersetzer.erzeugeButton( EINFUEGEN );
        einfuegen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                gewaehlteBoniModel.addElement( bonus.getValue() );
                pruefen();
            }
        });

        final JPanel spinnerPanel = new JPanel(
                new FormLayout( "P, 4dlu, P", "P" ) );

        final CellConstraints cc = new CellConstraints();

        spinnerPanel.add( bonus, cc.xy( 1, 1 ) );

        spinnerPanel.add( einfuegen, cc.xy( 3, 1) );

        final String spalten = "7dlu, F:P:G, 4dlu, F:P:G, 7dlu";

        final String zeilen;

        if ( Modus.PROFESSION.equals( modus ) ) {

            zeilen = "7dlu, P, 4dlu, P, 4dlu, P, 4dlu, F:P:G, " +
            		 "4dlu, P, 4dlu, P, 4dlu, F:P:G, 4dlu, P, 7dlu";

        } else {

            zeilen = "7dlu, P, 4dlu, P, 4dlu, F:P:G, " +
                     "4dlu, P, 4dlu, P, 4dlu, F:P:G, 4dlu, P, 7dlu";
        }

        final FormLayout layout = new FormLayout( spalten, zeilen );

        final JPanel inhalt = new JPanel( layout );

        if ( Modus.PROFESSION.equals( modus ) ) {

            inhalt.add( Util.erzeugeScrollPane( talente, 150, 200 ),
                        cc.xywh( 2, 2, 1, 13 ) );

            inhalt.add( uebernehmen, cc.xyw( 2, 16, 3 ) );

        } else {

            inhalt.add( Util.erzeugeScrollPane( talente, 150, 200 ),
                        cc.xywh( 2, 2, 1, 11 ) );

            inhalt.add( uebernehmen, cc.xyw( 2, 14, 3 ) );

        }

        int y = 0;

        inhalt.add( uebersetzer.erzeugeSeparator( TALENTE ), cc.xy( 4, y+=2 ) );

        inhalt.add( leittalent, cc.xy( 4, y+=2 ) );

        if ( Modus.PROFESSION.equals( modus ) ) {

            inhalt.add( entdecker, cc.xy( 4, y+=2 ) );
        }

        inhalt.add( Util.erzeugeScrollPane( gewaehlteTalenteListe, 150, 40 ),
                    cc.xy( 4, y+=2 ) );

        inhalt.add( uebersetzer.erzeugeSeparator( BONI ), cc.xy( 4, y+=2 ) );

        inhalt.add( spinnerPanel, cc.xy( 4, y+=2 ) );

        inhalt.add( Util.erzeugeScrollPane( gewaehlteBoniListe, 150, 20 ),
                    cc.xy( 4, y+=2 ) );
        
        final JScrollPane scrollPane = Util.erzeugeScrollPane( inhalt );
        resetter.einfuegen( scrollPane );
        
        setLayout( new BorderLayout() );
        
        add( scrollPane, BorderLayout.CENTER );
    }
    
    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }
    
    private void pruefen() {

        final int anzahlTalente = gewaehlteTalenteModel.getSize();

        final int anzahlBoni = gewaehlteBoniModel.getSize();

        if ( anzahlTalente < anzahlBoni ) {

            gewaehlteBoniListe.setBackground( Util.FEHLERFARBE );
            gewaehlteBoniListe.setToolTipText( ZU_VIELE_BONI );

        } else {

            gewaehlteBoniListe.setBackground(
                    gewaehlteTalenteListe.getBackground() );
            gewaehlteBoniListe.setToolTipText( null );
        }

        uebernehmen.setEnabled( ( anzahlTalente > 0 )
                                && ( anzahlBoni > 0 )
                                && ( anzahlTalente >= anzahlBoni ) );
    }

    /** Loescht die Eintraege im Panel. */
    public void eingabenLoeschen() {

        gewaehlteBoniModel.clear();

        gewaehlteTalenteModel.clear();
    }
}
