/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.sonderfertigkeiten.voraussetzungen;

import java.util.Arrays;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.talente.Talent;

class VoraussetzungTalent implements Voraussetzung {
    
    /** Das Talent in dem die Mindeststufe erreicht werden muss. */
    private final Talent talent;
    
    /** Die Mindeststufe die in dem Talent erreicht werden muss. */
    private final int mindeststufe;

    /**
     * @param dasTalent in dem die Mindeststufe erreicht werden muss.
     * @param dieMindeststufe die in dem Talent erreicht werden muss.
     */
    VoraussetzungTalent(
            final Talent dasTalent,
            final int dieMindeststufe ) {
        
        talent = dasTalent;
        
        mindeststufe = dieMindeststufe;
    }

    /**
     * @return das Talent.
     */
    public Collection<Object> getAbhaengigkeiten() {
        return Arrays.asList( (Object) talent );
    }
    
    public KonfliktBeschreibung pruefen(
            final Held held, 
            final SonderfertigkeitWert wert) {
        
        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();
        
        if ( ! held.hat( talent )
                || held.getStufe( talent ) < mindeststufe ) {
            
            beschreibung.einfuegen( String.format( 
                    I18n.i18n( "VoraussetzungTalent" ),
                    talent.getName(),
                    mindeststufe ) );
            
        }
        
        return beschreibung;
    }

}
