/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile.anpasser;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;
import de.bernhardjung.dsaprogramm.zauber.Zauber;
import de.bernhardjung.dsaprogramm.zauber.ZauberWert;

public class AnpasserUebernatuerlicheBegabung implements Anpasser {

    /**
     * Aktivert den gewaehlten Zauber.
     */
    public void eingefuegt( final Held held, final VorNachteilWert wert ) {

        if ( held.wirdErschaffen() ) {

            final Zauber zauber =  (Zauber) wert.getVariante().getElement();

            held.einfuegenVorgegeben( new ZauberWert( zauber, 3 ) );
        }
    }

    /**
     * Deaktivert den gewaehlten Zauber.
     */
    public void entfernt( final Held held, final VorNachteilWert wert ) {

        if ( held.wirdErschaffen() ) {

            final Zauber zauber = (Zauber) wert.getVariante().getElement();

            held.entfernenVorgegeben( new ZauberWert( zauber ) );
        }
    }

    public void stufeGeaendert( final Held held, final VorNachteilWert wert ) {
        // nichts zu tun.
    }

    /**
     * Nimmt dem Held den alten Zauber und aktivert den neuen Zauber.
     */
    public void geaendertVariante(
            final Held held,
            final Object alteVariante,
            final Object neueVariante ) {

        if ( held.wirdErschaffen() ) {

            final Zauber bisherigerZauber = (Zauber) alteVariante;

            final ZauberWert bisherigerWert = new ZauberWert( bisherigerZauber );

            final Zauber neuerZauber = (Zauber) neueVariante;

            final int stufe = held.getWert( bisherigerWert ).getStufe();

            held.entfernenVorgegeben( bisherigerWert );

            held.einfuegenVorgegeben( new ZauberWert( neuerZauber, 3 ) );

            held.einfuegen( new ZauberWert( neuerZauber, stufe - 3 ) );
        }
    }

}
