/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile.kosten;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

import java.util.Arrays;
import java.util.Collection;

class KostenBesondererBesitz implements Kosten {

    /** Die Kosten fuer Helden mit dem Vorteil Adlige Abstammung. */
    static final int GP_KOSTEN_ADLIG = 3;

    /** Die Kosten die normalerweise gezahlt werden muessen. */
    static final int GP_KOSTEN_NORMAL = 7;

    /** @return 0 */
    public int getAbbaukosten(
            final Held held,
            final VorNachteilWert wert,
            final Lernmethode lernmethode ) {
        return 0;
    }

    /** {@inheritDoc} */
    public int getGpKosten() {
        return GP_KOSTEN_NORMAL;
    }

    /** {@inheritDoc} */
    public int getGpKosten( final VorNachteilWert wert ) {
        return GP_KOSTEN_NORMAL;
    }

    /** @return Adlige Abstammung. */
    public Collection<Object> getAbhaengigkeiten() {
        return Arrays.asList( (Object) VorNachteil.ADLIGE_ABSTAMMUNG );
    }

    /** {@inheritDoc} */
    public int getGpKosten(
            final Held held,
            final VorNachteilWert wert ) {

        if ( held.hatVorgegeben( wert ) ) {

            return 0;
        }

        if ( held.hat( VorNachteil.ADLIGE_ABSTAMMUNG ) ) {

            return GP_KOSTEN_ADLIG;

        } else {

            return GP_KOSTEN_NORMAL;

        }
    }

    /** @return 0 */
    public int getSenkungskosten(final Lernmethode lernmethode, final VorNachteilWert wert ) {
        return 0;
    }

    /** @return {@code false} */
    public final boolean istSenkbar( final VorNachteilWert wert ) {
        return false;
    }
}
