/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile.kosten;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

import java.util.Arrays;
import java.util.Collection;

class KostenZV implements Kosten {

    /** Die GP-Kosten fuer Zauberer. */
    private final int gpKostenZauberer;

    /** Die GP-Kosten fuer Viertelzauberer. */
    private final int gpKostenViertelzauberer;

    /**
     * @param gpKostenZ die GP-Kosten fuer Zauberer.
     * @param gpKostenV die GP-Kosten fuer Viertelzauberer.
     */
    KostenZV( final int gpKostenZ, final int gpKostenV ) {

        gpKostenZauberer = gpKostenZ;

        gpKostenViertelzauberer = gpKostenV;
    }

    /** {@inheritDoc} */
    public int getAbbaukosten(
            final Held held,
            final VorNachteilWert wert,
            final Lernmethode lernmethode ) {

        if ( lernmethode == Lernmethode.FREI )
            return 0;

        if ( held.hat( VorNachteil.VOLLZAUBERER ) ) {

            return -100 * gpKostenZauberer;

        } else {

            return -100 * gpKostenViertelzauberer;

        }
    }

    /** @return eine leere Liste. */
    public Collection<Object> getAbhaengigkeiten() {
        return Arrays.asList(
                (Object) VorNachteil.VOLLZAUBERER,
                VorNachteil.VIERTELZAUBERER );
    }

    /** {@inheritDoc} */
    public int getGpKosten() {
        return Math.max( gpKostenViertelzauberer, gpKostenZauberer );
    }

    /** {@inheritDoc} */
    public int getGpKosten(final VorNachteilWert wert) {
        return getGpKosten();
    }

    /** {@inheritDoc} */
    public int getGpKosten( final Held held, final VorNachteilWert wert) {

        if ( held.hatVorgegeben( wert ) ) {
            return 0;
        }

        if ( held.hat( VorNachteil.VOLLZAUBERER ) ) {

            return gpKostenZauberer;

        } else {

            return gpKostenViertelzauberer;

        }
    }

    /** @return 0 */
    public int getSenkungskosten(final Lernmethode lernmethode, final VorNachteilWert wert ) {
        return 0;
    }

    /** @return {@code false} */
    public final boolean istSenkbar( final VorNachteilWert wert ) {
        return false;
    }

}
