/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile.voraussetzungen;

import java.util.Arrays;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

class VoraussetzungEigenschaft extends Voraussetzung {
    
    /** 
     * Die Eigenschaft in welcher der Held die Mindeststufe erreichen
     * muss.
     */
    private final Eigenschaft eigenschaft;
    
    /**
     * Die Stufe die der Held in der Eigenschaft mindestens besitzen muss.
     */
    private final int mindeststufe;
    
    VoraussetzungEigenschaft( 
            final Eigenschaft dieEigenschaft,
            final int dieMindeststufe ) {
        
        eigenschaft = dieEigenschaft;
        
        mindeststufe = dieMindeststufe;
    }

    /** 
     * @return die vorausgesetzte Eigenschaft.
     */
    public Collection<Object> getAbhaengigkeiten() {
        return Arrays.asList( (Object) eigenschaft );
    }

    public KonfliktBeschreibung pruefen(
            final Held held, 
            final VorNachteilWert wert) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();
        
        final int stufe = held.getStufe( eigenschaft );
        
        if ( stufe < mindeststufe ) {
            
            beschreibung.einfuegen( String.format(
                    I18n.i18n( "VoraussetzungEigenschaft" ),
                    eigenschaft.getKuerzel(),
                    mindeststufe ) );
            
        }
        
        return beschreibung;
    }

}
